Package: SteadyStateBVAR
Title: Bayesian Vector Autoregressions with Steady-State Priors
Version: 0.1.0
Authors@R: 
    person("Mark", "Becker", email = "mark.jw.becker@gmail.com", role = c("aut", "cre", "cph"))
Description: Provides estimation of Bayesian vector autoregression
    (BVAR) models with steady-state priors via 'Stan', along with
    functions for unconditional and conditional forecasting, as well as impulse
    response analysis. For details on the steady-state BVAR model
    see Villani (2009) <doi:10.1002/jae.1065>.
License: GPL (>= 3)
URL: https://github.com/markjwbecker/SteadyStateBVAR,
        https://markjwbecker.github.io/SteadyStateBVAR/
Depends: R (>= 3.6.0)
Imports: graphics, grDevices, methods, MTS (>= 1.2.1), Rcpp (>=
        0.12.0), RcppParallel (>= 5.0.1), rstan (>= 2.32.7), rstantools
        (>= 2.6.0), stats, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 8.0.0
Biarch: true
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
SystemRequirements: GNU make
BugReports: https://github.com/markjwbecker/SteadyStateBVAR/issues
NeedsCompilation: yes
Packaged: 2026-07-17 10:10:31 UTC; markj
Author: Mark Becker [aut, cre, cph]
Maintainer: Mark Becker <mark.jw.becker@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-24 10:30:10 UTC

# Generated by roxygen2: do not edit by hand

S3method(summary,bvar)
export(IRF)
export(bvar)
export(conditional_forecast)
export(fit)
export(forecast)
export(ppi)
export(priors)
export(restrict_beta)
export(setup)
export(stochastic_volatility_plot)
import(Rcpp)
import(methods)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,title)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
importFrom(stats,embed)
importFrom(stats,frequency)
importFrom(stats,is.ts)
importFrom(stats,median)
importFrom(stats,plot.ts)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,ts.plot)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(SteadyStateBVAR, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# SteadyStateBVAR

<!-- badges: start -->

[![Codecov test
coverage](https://codecov.io/gh/markjwbecker/steady_state_bvar/graph/badge.svg)](https://app.codecov.io/gh/markjwbecker/steady_state_bvar)
[![R-CMD-check](https://github.com/markjwbecker/SteadyStateBVAR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/markjwbecker/SteadyStateBVAR/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

With this package, the user can estimate the steady-state BVAR model of
Villani (2009). The steady-state BVAR is simply a BVAR rewritten in
mean-adjusted form. The benefit of the mean-adjusted parametrization is
that it allows the user to specify prior beliefs about the unconditional
mean, or *steady state* of the VAR system. The model has proven very
useful for forecasting of macroeconomic variables, and is routinely used
in many central banks and other finanicial institutions (Gustafsson and
Villani, 2025).

After estimation, the user can produce forecasts (unconditional and
conditional) and impulse response functions (orthogonalized and
generalized). The goal of `SteadyStateBVAR` is to use modern Bayesian
tools (Stan) to: i) estimate the model as specified in the original
paper, and ii) extend the model in different ways. Previously,
extensions of the model seemed to be limited by what Mattias Villani had
time to derive.

See for example Clark (2011), which extends the steady-state BVAR model
to include Random Walk stochastic volatility: “*In a methodological
sense, this paper extends the estimator of Villani (2009) to include
stochastic volatility.*” Later on, he writes: “*(Special thanks are due
to Mattias Villani for providing the formulas for posterior means and
variances of* $\Pi$ *and* $\Psi$*, which generalize the
constant-variance formulas of Villani 2009.)*”

Another example is Dieppe, Legrand, and van Roye (2016): “*Villani
(2009) only provides derivation in the case of the normal-diffuse prior
distribution, so the incoming analysis will be restricted to this
case.*”

Times are different now, and with the help of Stan, we are essentially
limited only by our imagination. At the time of writing, the package
provides three versions of the steady-state BVAR model: i) the
homoscedastic model, i.e. the original model in Villani (2009), ii) the
Random Walk stochastic volatility model, i.e. the one in Clark (2011),
and iii) an AR(1) stochastic volatility model.

## Installation

You can install SteadyStateBVAR with:

``` r
install.packages("SteadyStateBVAR")
```

You can also install the development version of SteadyStateBVAR from
[GitHub](https://github.com/) with:

``` r
# install.packages("pak")
pak::pak("markjwbecker/SteadyStateBVAR")
```

## Introduction

The steady-state BVAR($p$) model (Villani, 2009) is

$$y_t = \Psi d_t + \Pi_1(y_{t-1}-\Psi d_{t-1})+\dots+\Pi_p(y_{t-p}-\Psi d_{t-p})+u_t$$

where $y_t$ is a $k$-dimensional vector of endogenous variables (time
series) at time $t$, $d_t$ is a $q$-dimensional vector of deterministic
(exogenous) variables at time $t$, and the (reduced-form) innovations
are $u_t \overset{\text{iid}}{\sim} \mathrm{N_k}(0,\Sigma_u)$. Here
$\Pi_\ell$ for $\ell=1,\dots,p$ is a $(k \times k)$ autoregressive
parameter matrix, and $\Psi$ is a $(k \times q)$ steady-state parameter
matrix. Now

$$\mathrm{E}(y_t)=\mu_t=\Psi d_t$$

is the unconditional mean, or the **steady state**, of the process.
Since long-horizon forecasts from stationary VARs converge to the
unconditional mean (steady state), it is naturally very important from a
forecasting perspective to obtain precise inference on $\Psi$. Note that
the current version of this package only allows for $d_t$ to contain
either a constant, a constant and a dummy variable, or a constant and a
time trend.

We may stack the (transposed) $\Pi_i$ matrices in the $(kp \times k)$
matrix $\beta$

$$\beta=
\begin{bmatrix}
\Pi'_1 \\ 
\vdots  \\
\Pi'_p
\end{bmatrix}$$

We can then rewrite the model as a nonlinear regression (Karlsson, 2013)

$$y_t' =d_t'\Psi' + \left[w_t'-q_t'(I_p \otimes \Psi') \right]\beta +u_t'$$

where $w_t'=(y_{t-1}',\dots,y_{t-p}')$ is a $kp$-dimensional vector of
lagged endogenous variables and $q_t'=(d_{t-1}',\dots,d_{t-p}')$ is a
$qp$-dimensional vector of lagged deterministic (exogenous) variables,
$I_p$ is the $(p \times p)$ identity matrix and $\otimes$ denotes the
Kronecker product. This is how the likelihood is written in the Stan
code. The goal is to estimate $\beta, \Psi$, and $\Sigma_u$, and as such
priors are needed. Following Villani (2009), prior independence between
$\beta, \Psi$ and $\Sigma_u$ is assumed. For $\beta$, the Minnesota
prior is used

$$\mathrm{vec}(\beta) \sim \mathrm{N}_{kpk} \left[\theta_\beta,\Omega_\beta\right]$$

The prior means (the elements of $\theta_\beta$) are set to

$$\begin{aligned}
\mathrm{E}\left(\Pi_{\ell}^{(i,j)}\right)&=
\begin{cases}
\kappa & \text{if } \ell = 1 \ \mathrm{and} \ i = j \\
0 & \mathrm{otherwise}
\end{cases}\\
\kappa&=
\begin{cases}
\kappa^{level} & \text{if } \mathrm{variable} \ i \ \mathrm{is in level} \\
\kappa^{\Delta} & \text{if }\mathrm{variable} \ i \ \mathrm{is in difference}
\end{cases}\\
\end{aligned}$$

Here, the autoregressive coefficient $\Pi_{\ell}^{(i,j)}$ is element
$\left(i,j\right)$ of $\Pi_{\ell}$ for $\ell=1,\dots,p$. As such, the
Minnesota prior sets all prior means for the elements in $\beta$ to $0$,
except for the elements that relate to the first own lags of the
variables, which are set to $\kappa$. If variable $i$ is in level
(e.g. nominal interest rate), then $\kappa=\kappa^{level}$, and typical
choices for $\kappa^{level}$ are $1$ or $0.9$. Evaluating the equations
at their prior means, equation $i$ becomes a random walk if
$\kappa^{level}=1$ and a persistent stationary AR(1) process if
$\kappa^{level}=0.9$. Since the steady state only exists if the process
is stationary, $0.9$ is recommended for the steady-state BVAR. If
variable $i$ is in difference (e.g. output growth), then
$\kappa=\kappa^{\Delta}$, and the most common choice for
$\kappa^{\Delta}$ is $0$, i.e. equation $i$ becomes (when evaluating it
at its prior means) a random walk expressed in first differences. If a
differenced variable still shows some degree of persistence (can be
examined with an ACF plot), a suitable value for $\kappa^{\Delta}$ can
be (for example) $0.6$ instead of $0$. Moving on to the prior variances,
$\Omega_\beta$ is a diagonal matrix containing the prior variances for
the elements in $\beta$. They are specified as

$$\mathrm{Var}\left(\Pi_{\ell}^{(i,j)}\right)=
\begin{cases}
\left(\frac{\lambda_1}{\ell^{\lambda_3}}\right)^2 & \text{if } i = j \\
\left(\frac{\lambda_1 \lambda_2\sigma_i}{\ell^{\lambda_3}\sigma_j}\right)^2& \text{if } i \neq j
\end{cases}$$

Here $\lambda_1$, $\lambda_2$, and $\lambda_3$ are scalar
hyperparameters known as the overall tightness, the cross-equation
tightness and the lag decay rate. Furthermore, $\sigma_i^2$ is the
$(i,i)$:th element of $\Sigma_u$, which we do not know and therefore
replace with an estimate. In this package, it is replaced by the least
squares residual variance from a univariate autoregression for variable
$i$ with $p$ lags (including the constant and dummy/trend variable if
applicable). Moving on to $\Psi$, which contains the steady-state
parameters, the prior is

$$\mathrm{vec}(\Psi) \sim \mathrm{N}_{kq}\left[\theta_\Psi,\Omega_\Psi\right]$$

This is the core of the steady-state BVAR model. In $\theta_\Psi$, we
specify our prior beliefs about the location of the steady state, and in
$\Omega_\Psi$, which we assume to be a diagonal matrix, we specify our
degree of certainty in those prior beliefs. Finally, the prior for
$\Sigma_u$ is the usual non-informative Jeffreys prior

$$p(\Sigma_u) \propto\left|\Sigma_u \right|^{-(k+1)/2}$$

Alternatively, a proper inverse-Wishart prior can be used (Karlsson,
2013)

$$\Sigma_u \sim \mathrm{IW}(V,m)$$

where $V$ is the scale matrix and $m$ is the number of degrees of
freedom. As mentioned, this package also allows for stochastic
volatility (Random Walk or AR(1) specifications), where the covariance
matrix varies over time, i.e. we have $\Sigma_{u,t}$ (see `?bvar` for
more details).

## Example

To estimate the model in Section 4.1 of Villani (2009), produce
(un)conditional forecasts and perform impulse response analysis, simply
run the following code:

``` r
library(SteadyStateBVAR)
data("Villani2009")
yt <- Villani2009

#hold out last two observations to facilitate comparisons
#to Figures 1-3 in Villani (2009)
yt <- ts(yt[1:102, ], start = start(yt), frequency = frequency(yt))

bvar_obj <- bvar(data = yt)

#Use a dummy to model Sweden’s change in monetary policy in the 1990s
#(move to inflation targeting and flexible exchange rate)
bp <- which(time(yt) == 1992.75) #breakpoint
dummy_variable <- c(rep(1,bp), rep(0,nrow(yt)-bp))

bvar_obj <- setup(bvar_obj,
                  p=4,
                  deterministic = "constant_and_dummy",
                  dummy = dummy_variable)

lambda_1 <- 0.2 #overall tightness
lambda_2 <- 0.5 #cross-equation tightness
lambda_3 <- 1.0 #lag decay rate

#fol_pm = first own lag prior means
fol_pm=c(0,   #delta y_f
         0,   #pi_f
         0.9, #i_f
         0,   #delta y
         0,   #pi
         0.9, #i
         0.9  #q
         )

#95% prior probability intervals (normal distribution)
#See Table I in Villani (2009)
#These are the "steady-state priors"
#psi_1 = Psi col 1
#psi_2 = Psi col 2

theta_Psi <- 
  c(
  ppi( 2.00,  3.00,  annualized_growthrate=TRUE)$mean,   #psi_1: delta y_f
  ppi( 1.50,  2.50,  annualized_growthrate=TRUE)$mean,   #psi_1: pi_f
  ppi( 4.50,  5.50,  annualized_growthrate=FALSE)$mean,  #psi_1: i_f
  ppi( 2.00,  2.50,  annualized_growthrate=TRUE)$mean,   #psi_1: delta y
  ppi( 1.70,  2.30,  annualized_growthrate=TRUE)$mean,   #psi_1: pi
  ppi( 4.00,  4.50,  annualized_growthrate=FALSE)$mean,  #psi_1: i
  ppi( 3.85,  4.00,  annualized_growthrate=FALSE)$mean,  #psi_1: q
  ppi(-1.00,  1.00,  annualized_growthrate=TRUE)$mean,   #psi_2: delta y_f
  ppi( 1.50,  2.50,  annualized_growthrate=TRUE)$mean,   #psi_2: pi_f
  ppi( 1.50,  2.50,  annualized_growthrate=FALSE)$mean,  #psi_2: i_f
  ppi(-1.00,  1.00,  annualized_growthrate=TRUE)$mean,   #psi_2: delta y
  ppi( 4.30,  5.70,  annualized_growthrate=TRUE)$mean,   #psi_2: pi
  ppi( 3.00,  5.50,  annualized_growthrate=FALSE)$mean,  #psi_2: i
  ppi(-0.50,  0.50,  annualized_growthrate=FALSE)$mean   #psi_2: q
  )

Omega_Psi <- 
  diag(
  c(
  ppi( 2.00,  3.00,  annualized_growthrate=TRUE)$var,    #psi_1: delta y_f
  ppi( 1.50,  2.50,  annualized_growthrate=TRUE)$var,    #psi_1: pi_f
  ppi( 4.50,  5.50,  annualized_growthrate=FALSE)$var,   #psi_1: i_f
  ppi( 2.00,  2.50,  annualized_growthrate=TRUE)$var,    #psi_1: delta y
  ppi( 1.70,  2.30,  annualized_growthrate=TRUE)$var,    #psi_1: pi
  ppi( 4.00,  4.50,  annualized_growthrate=FALSE)$var,   #psi_1: i
  ppi( 3.85,  4.00,  annualized_growthrate=FALSE)$var,   #psi_1: q
  ppi(-1.00,  1.00,  annualized_growthrate=TRUE)$var,    #psi_2: delta y_f
  ppi( 1.50,  2.50,  annualized_growthrate=TRUE)$var,    #psi_2: pi_f
  ppi( 1.50,  2.50,  annualized_growthrate=FALSE)$var,   #psi_2: i_f
  ppi(-1.00,  1.00,  annualized_growthrate=TRUE)$var,    #psi_2: delta y
  ppi( 4.30,  5.70,  annualized_growthrate=TRUE)$var,    #psi_2: pi
  ppi( 3.00,  5.50,  annualized_growthrate=FALSE)$var,   #psi_2: i
  ppi(-0.50,  0.50,  annualized_growthrate=FALSE)$var    #psi_2: q
  )
  )

bvar_obj <- priors(bvar_obj,
                   lambda_1,
                   lambda_2,
                   lambda_3,
                   fol_pm,
                   theta_Psi,
                   Omega_Psi,
                   Jeffreys=TRUE) #FALSE for uninformative inverse-Wishart

p <- bvar_obj$setup$p
k <- bvar_obj$setup$k
kf <- 3 #first three variables in yt are foreign

restriction_matrix <- matrix(1, k*p, k)

for(i in 1:p){
  rows <- ((i-1)*k + kf + 1) : (i*k)
  cols <- 1:kf
  restriction_matrix[rows, cols] <- 0
}
print(restriction_matrix)
#block exogeneity for foreign variables
bvar_obj <- restrict_beta(bvar_obj, restriction_matrix)

H <- 12 #forecast horizon
(d_pred <- cbind(rep(1, 12), 0)) #future d_t values


#fit the model
bvar_obj <- fit(bvar_obj,
                H = H,
                d_pred = d_pred,
                iter = 10000,
                warmup = 2500,
                chains = 2,
                cores = 2)

#posterior summaries
summary(bvar_obj , stat = "mean")

#you can look at the stanfit object directly
stan_fit <- bvar_obj$fit$stan
print(stan_fit)

#unconditional forecasts
#see last forecasts in Figures 1-3 in Villani (2009)
fcst <- forecast(bvar_obj,
                 pi = 0.95, #pi = prediction interval
                 fcst_type = "mean",
                 growth_rate_idx = c(4,5), #convert QoQ forecasts to YoY
                 plot_idx = c(4,5,6))

#conditional forecasts
#Toy scenario: inflation gets really high
#What will happen to domestic interest rate?
conditions <- data.frame(
              var        = rep(5,12),
              horizon    = rep(1:12),
              value      = c(1.0,1.5,2.0,1.8,
                             1.5,1.2,1.0,1.0,
                             rep(0.5,4)) #QoQ scale for inflation here
              )
              
cond_fcst <- conditional_forecast(bvar_obj,
                    conditions,
                    pi=0.68,
                    fcst_type = "mean",
                    plot_idx = c(5,6),
                    growth_rate_idx = c(5)) #convert QoQ forecasts to YoY

#impulse response analysis
irf <- IRF(bvar_obj,
           H=20,
           response=5,#inflation
           shock=6, #interest rate
           type="median",
           method="OIRF",
           ci=0.68,
           growth_rate_idx=5) #YoY inflation instead of QoQ
```

## References

Clark, T. E. (2011). Real-time density forecasts from Bayesian vector
autoregressions with stochastic volatility. *Journal of Business &
Economic Statistics*, 29(3), pp. 327–341.

Dieppe, A., Legrand, R., and van Roye, B. (2016). The BEAR toolbox.
Working Paper Series, No. 1934. European Central Bank.

Gustafsson, O., and Villani, M. (2025). Variational inference for
steady-state BVARs. arXiv preprint arXiv:2506.09271.

Karlsson, S. (2013). Forecasting with Bayesian vector autoregression.
In: Elliott, G. and Timmermann, A. (eds), *Handbook of Economic
Forecasting*. Elsevier B.V., Vol. 2, Part B, pp. 791–897.

Villani, M. (2009). Steady-state priors for vector autoregressions.
*Journal of Applied Econometrics*, 24(4), pp. 630–650.

#' Impulse Response Functions for a fitted steady-state BVAR model
#'
#' Computes and plots impulse response functions (IRFs) from a fitted
#' steady-state \code{bvar} object. Supports both orthogonalized (OIRF) and generalized
#' (GIRF) impulse responses, with optional conversion to annual growth rates.
#'
#' @param x A steady-state \code{bvar} object that has been passed through \code{\link{fit}}.
#' @param H Integer. The forecast horizon for the IRF. Default \code{16}.
#' @param response Integer. Index of the response variable to plot. If
#'   \code{NULL} (default), all responses are plotted.
#' @param shock Integer. Index of the shock variable to plot. If \code{NULL}
#'   (default), all shocks are plotted.
#' @param type Character. Whether to use \code{"median"} or \code{"mean"} as
#'   the point estimate. Default \code{"median"}.
#' @param method Character. The IRF method: \code{"OIRF"} for orthogonalized
#'   or \code{"GIRF"} for generalized impulse responses. Default \code{"OIRF"}.
#' @param ci Numeric. The credible interval width. Default \code{0.95}, i.e. 95%.
#' @param t Integer. Time index for the covariance matrix when using stochastic
#'   volatility models. If \code{NULL} (default), the last time \code{t} is used.
#' @param growth_rate_idx Integer vector. Indices of variables for which the impulse
#'   response is converted from a quarterly or monthly log first difference to an
#'   annual growth rate response, i.e. \eqn{\ln x_{t} - \ln x_{t-f}}, where \eqn{f} is
#'   the frequency of the data (4 for quarterly, 12 for monthly).
#'   Only suitable for variables specified as \eqn{\ln x_{t} - \ln x_{t-1}}, i.e.
#'   \code{diff(log(x))} or \code{100*diff(log(x))}. Computed by summing up to
#'   \eqn{f} periods of the impulse response, treating the response in periods
#'   prior to the shock as zero. Default is \code{NULL}.
#'
#' @return Invisibly returns a list with three arrays: the point estimate IRF, \code{lower}, and
#'   \code{upper} credible bounds, each of dimension \code{k x k x (H+1)}.
#' @export
#'
#' @examples
#' \donttest{
#' #homoscedastic with Jeffreys prior
#' yt <- matrix(rnorm(50), 25, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    Jeffreys = TRUE,
#'                    SV = FALSE,
#'                    SV_type = NULL,
#'                    SV_priors = NULL)
#'                    
#' bvar_obj <- fit(bvar_obj,
#'                 H = 8,
#'                 d_pred = matrix(rep(1,8)),
#'                 iter = 200,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1)
#'                 
#' (IRF(bvar_obj))
#' }
IRF <- function(x, H = 16, response = NULL, shock = NULL,
                type = c("median", "mean"), method = c("OIRF", "GIRF"),
                ci = 0.95, t = NULL, growth_rate_idx = NULL) {
  
  method <- match.arg(method)
  type   <- match.arg(type)
  freq   <- frequency(x$data)
  
  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par), add = TRUE)
  
  compute_OIRF <- function(A, Sigma, e, N) {
    k            <- nrow(Sigma)
    OIRF_matrix  <- matrix(NA, k * k, N + 1)
    count        <- 1
    P            <- t(chol(Sigma))
    for (j in 1:k) {
      for (i in 1:k) {
        for (n in 0:N) {
          OIRF_matrix[count, n + 1] <- (A[[n + 1]] %*% P %*% e[[j]])[i]
        }
        count <- count + 1
      }
    }
    return(OIRF_matrix)
  }
  
  compute_GIRF <- function(A, Sigma, e, N) {
    k           <- nrow(Sigma)
    GIRF_matrix <- matrix(NA, k * k, N + 1)
    count       <- 1
    for (j in 1:k) {
      for (i in 1:k) {
        for (n in 0:N) {
          GIRF_matrix[count, n + 1] <- (Sigma[j, j]^(-1/2) * A[[n + 1]] %*% Sigma %*% e[[j]])[i]
        }
        count <- count + 1
      }
    }
    return(GIRF_matrix)
  }
  
  k         <- x$setup$k
  p         <- x$setup$p
  var_names <- if (is.null(colnames(x$data))) paste0("y_", 1:k) else colnames(x$data)
  
  stan_draws <- rstan::extract(x$fit$stan)
  N_draws    <- dim(stan_draws$beta)[1]
  
  irf_array <- array(NA, dim = c(N_draws, k, k, H + 1))
  
  e <- vector("list", k)
  for (nE in 1:k) {
    tmp      <- rep(0, k)
    tmp[nE]  <- 1
    e[[nE]]  <- tmp
  }
  
  for (draw in 1:N_draws) {
    Phi <- t(stan_draws$beta[draw, , ])
    if (length(dim(stan_draws$Sigma_u)) == 4) {
      t_use <- if (is.null(t)) dim(stan_draws$Sigma_u)[2] else t - x$setup$p
      Sigma  <- stan_draws$Sigma_u[draw, t_use, , ]
    } else {
      Sigma <- stan_draws$Sigma_u[draw, , ]
    }
    
    Psi  <- MTS::VARpsi(Phi, lag = H)$psi
    kk   <- ncol(Psi) / nrow(Psi)
    A    <- vector("list", length = kk)
    for (nA in 1:kk) {
      col_idx  <- ((nA - 1) * nrow(Psi) + 1):(nA * nrow(Psi))
      A[[nA]]  <- Psi[, col_idx]
    }
    
    tmp_IRF <- if (method == "OIRF") compute_OIRF(A, Sigma, e, H) else compute_GIRF(A, Sigma, e, H)
    
    for (j in 1:k) {
      for (i in 1:k) {
        irf_array[draw, i, j, ] <- tmp_IRF[(j - 1) * k + i, ]
      }
    }
  }
  
  alpha     <- 1 - ci
  m_irf     <- apply(irf_array, c(2, 3, 4), type)
  lower_irf <- apply(irf_array, c(2, 3, 4), quantile, probs = alpha / 2)
  upper_irf <- apply(irf_array, c(2, 3, 4), quantile, probs = 1 - alpha / 2)
  
  if (!is.null(growth_rate_idx)) {
    transform_irf <- function(irf_mat, freq) {
      dims     <- dim(irf_mat)
      irf_yoy  <- array(NA, dim = dims)
      for (i in 1:dims[1]) {
        for (j in 1:dims[2]) {
          if (i %in% growth_rate_idx) {
            irf      <- irf_mat[i, j, ]
            all_irf  <- c(rep(0, freq - 1), irf)
            tmp      <- numeric(dims[3])
            for (h in 1:dims[3]) {
              tmp[h] <- sum(all_irf[h:(h + freq - 1)])
            }
            irf_yoy[i, j, ] <- tmp
          } else {
            irf_yoy[i, j, ] <- irf_mat[i, j, ]
          }
        }
      }
      return(irf_yoy)
    }
    m_irf     <- transform_irf(m_irf, freq)
    lower_irf <- transform_irf(lower_irf, freq)
    upper_irf <- transform_irf(upper_irf, freq)
  }
  
  horizon    <- 0:H
  type_label <- if (type == "median") "Median" else "Mean"
  
  plot_single <- function(i, j) {
    max_abs    <- max(abs(c(lower_irf[i, j, ], upper_irf[i, j, ], m_irf[i, j, ])))
    ylim_range <- c(-max_abs, max_abs)
    ylab_label <- if (!is.null(growth_rate_idx) && i %in% growth_rate_idx) {
      paste0("Response: ", var_names[i], " (annual)")
    } else {
      paste0("Response: ", var_names[i])
    }
    
    plot(NA, xlim = c(0, H), ylim = ylim_range, type = "n",
         xlab = "Horizon", ylab = ylab_label, font.lab = 2)
    polygon(c(horizon, rev(horizon)),
            c(upper_irf[i, j, ], rev(lower_irf[i, j, ])),
            col = rgb(0, 0, 1, 0.2), border = NA)
    lines(horizon, m_irf[i, j, ], col = "blue", lwd = 2)
    abline(h = 0, col = "black", lty = 1)
    
    main_title <- if (isTRUE(x$priors$SV)) {
      paste0("Posterior ", type_label, " ", method, " (", round(ci * 100), "% CI)\nt=", t, "\nShock: ", var_names[j])
    } else {
      paste0("Posterior ", type_label, " ", method, " (", round(ci * 100), "% CI)\n\nShock: ", var_names[j])
    }
    title(main = main_title)
  }
  
  if (is.null(response) || is.null(shock)) {
    par(mfrow = c(k, k))
    for (j in 1:k) for (i in 1:k) plot_single(i, j)
    par(mfrow = c(1, 1))
  } else {
    plot_single(response, shock)
  }
  
  if (type == "median") {
    invisible(list(median_irf = m_irf, lower = lower_irf, upper = upper_irf))
  } else {
    invisible(list(mean_irf = m_irf, lower = lower_irf, upper = upper_irf))
  }
}

#' Prior Probability Interval for a Normal Distribution
#'
#' Calculates the mean and variance of a normal prior probability interval.
#' Given a lower and upper bound of a prior probability interval
#' this function recovers the implied normal prior parameters.
#' Useful for specifying informative priors
#' on the steady-state parameters (elements of \eqn{\Psi}).
#'
#' @param l Numeric. The lower bound of the prior probability interval.
#' @param u Numeric. The upper bound of the prior probability interval.
#' @param interval Numeric. The prior probability mass within the interval.
#'   Default \code{0.95}, i.e. 95%.
#' @param annualized_growthrate Logical. If \code{TRUE}, treats \code{l} and \code{u} as
#'   bounds on the annualized steady-state growth rate and calculates the implied mean
#'   and variance on the corresponding quarterly/monthly scale. Useful if you are working with
#'   a variable \code{diff(log(x))} (which may be scaled by 100). Default \code{FALSE}.
#' @param freq Integer. The data frequency (e.g. \code{4} for quarterly).
#'   Only used when \code{annualized_growthrate = TRUE}. Default \code{4}.
#'
#' @return A list with two elements: \code{mean} and \code{var}, giving the
#'   mean and variance of the implied normal distribution.
#' @export
#' 
#' @details
#' Consider a CPI variable \code{CPI <- data$CPI}, observed at quarterly frequency.
#' In the model, quarter-on-quarter inflation is used \code{x <- 100*diff(log(CPI))}.
#' Suppose the prior belief is that annualized steady-state inflation lies between 1.7 and 2.3
#' with 95% probability (mean 2). On the quarterly scale used by \code{x}, this corresponds to a
#' 95% interval of 0.425 to 0.575 (mean 0.5). Since it is typically more natural to elicit a prior
#' on the annualized scale, the \code{annualized_growthrate} argument performs this conversion.
#' See \code{vignette("Homoscedastic-steady-state-BVAR")} for usage in practice.
#' 
#'
#' @examples
#' ppi(l = 1.7, u = 2.3, interval = 0.95)
#' ppi(l = 1.7, u = 2.3, interval = 0.95, annualized_growthrate = TRUE, freq = 4)
ppi <- function(l, u, interval = 0.95, annualized_growthrate = FALSE, freq = 4) {
  
  if (l >= u) {
    stop("lower bound must be less than upper bound")
  }
  
  if (interval <= 0 || interval >= 1) {
    stop("interval must be between 0 and 1")
  }
  
  if (annualized_growthrate && freq <= 0) {
    stop("freq must be positive")
  }
  
  alpha <- 1 - interval
  z     <- qnorm(1 - alpha / 2)
  
  if (!annualized_growthrate) {
    mu    <- (u + l) / 2
    sigma <- (u - l) / (2 * z)
  } else {
    mu    <- (u + l) / 2 / freq
    sigma <- (u - l) / (2 * z) / freq
  }
  
  return(list(mean = mu, var = sigma^2))
}

test_that("ppi validates lower < upper", {
  expect_error(ppi(l = 3, u = 1), "lower bound must be less than upper")
})

test_that("ppi validates interval is between 0 and 1", {
  expect_error(ppi(l = 1, u = 3, interval = 1.5), "interval must be between 0 and 1")
  expect_error(ppi(l = 1, u = 3, interval = 0), "interval must be between 0 and 1")
})

test_that("ppi validates freq is positive", {
  expect_error(ppi(l = 1, u = 3, annualized_growthrate = TRUE, freq = -1), 
               "freq must be positive")
})

test_that("ppi returns list with mean and var", {
  result <- ppi(l = 1, u = 3)
  
  expect_type(result, "list")
  expect_named(result, c("mean", "var"))
  expect_type(result$mean, "double")
  expect_type(result$var, "double")
})

test_that("ppi calculates mean correctly", {
  result <- ppi(l = 1, u = 3, interval = 0.95)
  
  expect_equal(result$mean, 2)
})

test_that("ppi variance is always positive", {
  result <- ppi(l = 1, u = 3)
  expect_gt(result$var, 0)
})

test_that("ppi wider interval has larger variance", {
  result_narrow <- ppi(l = 2, u = 3)
  result_wide <- ppi(l = 1, u = 4)
  
  expect_lt(result_narrow$var, result_wide$var)
})

test_that("ppi with different confidence intervals", {
  result_68 <- ppi(l = 1, u = 3, interval = 0.68)
  result_95 <- ppi(l = 1, u = 3, interval = 0.95)
  result_99 <- ppi(l = 1, u = 3, interval = 0.99)
  
  expect_equal(result_68$mean, result_95$mean)
  expect_equal(result_95$mean, result_99$mean)
  
  # Lower confidence = larger variance (for same bounds)
  expect_gt(result_68$var, result_95$var)
  expect_gt(result_95$var, result_99$var)
})

test_that("ppi with annualized growth rate conversion", {
  result_annual <- ppi(l = 4, u = 8, annualized_growthrate = TRUE, freq = 4)
  
  expect_equal(result_annual$mean, 1.5)
})

test_that("ppi annualized conversion scales variance correctly", {
  result_level <- ppi(l = 4, u = 8, annualized_growthrate = FALSE)
  result_annual <- ppi(l = 4, u = 8, annualized_growthrate = TRUE, freq = 4)
  
  expect_equal(result_annual$var, result_level$var / 16)
})

test_that("ppi with custom frequency", {
  result_monthly <- ppi(l = 12, u = 24, annualized_growthrate = TRUE, freq = 12)
  result_quarterly <- ppi(l = 12, u = 24, annualized_growthrate = TRUE, freq = 4)
  
  expect_equal(result_monthly$mean, 1.5)
  expect_equal(result_quarterly$mean, 4.5)
  
  expect_lt(result_monthly$var, result_quarterly$var)
})

test_that("ppi symmetric intervals", {
  result_pos <- ppi(l = 0, u = 4)
  expect_equal(result_pos$mean, 2)
  
  result_shift <- ppi(l = 3, u = 7)
  expect_equal(result_shift$mean, 5)
})

test_that("ppi with very tight interval", {
  result <- ppi(l = 2.99, u = 3.01, interval = 0.95)
  
  expect_equal(result$mean, 3)
  expect_lt(result$var, 0.0001)
})

test_that("ppi with very wide interval", {
  result <- ppi(l = 0, u = 100, interval = 0.95)
  
  expect_equal(result$mean, 50)
  expect_gt(result$var, 100)
})

test_that("ppi realistic inflation prior", {
  result <- ppi(l = 1, u = 3, interval = 0.90)
  
  expect_equal(result$mean, 2)
  expect_gt(result$var, 0)
})

test_that("ppi realistic growth rate prior annualized", {
  result <- ppi(l = 2, u = 4, annualized_growthrate = TRUE, freq = 4)
  
  expect_equal(result$mean, 0.75)
})

#' Summarise a fitted steady-state BVAR model
#'
#' Computes and prints posterior summaries from a fitted steady-state \code{bvar} object.
#' The printed output depends on whether the model is homoscedastic or includes
#' stochastic volatility (RW or AR1 specification).
#'
#' @param object A steady-state \code{bvar} object that has been passed through
#'   \code{\link{fit}}.
#' @param pars Character vector of parameter names to include. If
#'   \code{NULL} (default), all available parameters are displayed.
#' @param stat Character. Posterior summary statistic to display:
#'   \code{"mean"} (default) or \code{"median"}.
#' @param t Integer. Time index for the innovation covariance matrix if stochastic volatility was estimated.
#'   If \code{NULL}, the latest available \code{t} is used.
#' @param ... Additional arguments (currently unused).
#'
#' @return Returns the input object invisibly.
#'
#' @details
#' The function summarises the following estimated parameters:
#' \itemize{
#'   \item \code{beta}: \eqn{kp \times k} VAR coefficient matrix
#'   \item \code{Psi}: \eqn{k \times q} steady-state parameter matrix
#'   \item \code{Sigma_u}: innovation covariance matrix (\eqn{k \times k} for homoscedastic,
#'         \eqn{T \times k \times k} for stochastic volatility)
#'   \item If Random Walk stochastic volatility:
#'    \itemize{
#'    \item \code{A}: \eqn{k \times k} lower triangular matrix with ones on the diagonal that describes
#'     the contemporaneous interaction of the endogenous variables
#'    \item \code{phi}: \eqn{k}-dimensional vector of log volatility innovation variances
#'    }
#'   \item If AR1 stochastic volatility:
#'    \itemize{
#'    \item \code{A}: \eqn{k \times k} lower triangular matrix with ones on the diagonal that describes
#'     the contemporaneous interaction of the endogenous variables
#'     \item \code{gamma_0}: \eqn{k}-dimensional vector of log volatility intercepts
#'     \item \code{gamma_1}: \eqn{k}-dimensional vector of log volatility slopes
#'     \item \code{Phi}: \eqn{k \times k} log volatility innovation covariance matrix
#'    }
#' }
#'
#' @method summary bvar
#' @export
#' @examples
#' \donttest{
#' yt <- matrix(rnorm(20), 10, 2)
#' bvar_obj <- bvar(data = yt)
#' bvar_obj <- setup(bvar_obj, p = 1, deterministic = "constant")
#' bvar_obj <- priors(bvar_obj,
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2))
#'
#' bvar_obj <- fit(bvar_obj,
#'                 H = 1,
#'                 d_pred = matrix(1),
#'                 iter = 100,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1)
#'
#' summary(bvar_obj)
#' }
summary.bvar <- function(object, pars = NULL, stat = "mean", t = NULL, ...) {
  
  if (is.null(object$setup)) stop("object must be passed through setup() first")
  if (is.null(object$priors)) stop("object must be passed through priors() first")
  if (is.null(object$fit)) stop("object must be passed through fit() first")
  
  stat <- match.arg(stat, c("mean", "median"))
  posterior <- rstan::extract(object$fit$stan)
  
  var_names <- colnames(object$data)
  if (is.null(var_names)) {
    var_names <- paste0("Var", 1:ncol(object$data))
  }
  
  is_sv <- !is.null(object$priors$SV) && object$priors$SV
  
  compute_stat <- function(arr, stat) {
    apply(arr, c(2, 3), if (stat == "mean") mean else median)
  }
  
  results <- list()
  
  # ---------------- beta ----------------
  if (!is.null(posterior$beta)) {
    beta <- compute_stat(posterior$beta, stat)
    
    var_rep <- rep(var_names, object$setup$p)
    lag_names <- paste0("l", rep(1:object$setup$p, each = object$setup$k))
    
    rownames(beta) <- paste0(var_rep, ".", lag_names)
    colnames(beta) <- var_names
    
    results$beta <- round(beta, 2)
  }
  
  # ---------------- Psi ----------------
  if (!is.null(posterior$Psi)) {
    Psi <- compute_stat(posterior$Psi, stat)
    rownames(Psi) <- var_names
    results$Psi <- round(Psi, 2)
  }
  
  # ---------------- Sigma_u ----------------
  if (!is.null(posterior$Sigma_u)) {
    
    if (is_sv) {
      
      N_original <- nrow(object$data)
      N_est <- N_original - object$setup$p
      
      if (is.null(t)) {
        t_est <- N_est
        t_display <- N_original
      } else {
        if (t < object$setup$p + 1 || t > N_original) {
          stop("t must be between ", object$setup$p + 1, " and ", N_original)
        }
        t_est <- t - object$setup$p
        t_display <- t
      }
      
      Sigma <- posterior$Sigma_u[, t_est, , ]
      Sigma <- compute_stat(Sigma, stat)
      
      rownames(Sigma) <- var_names
      colnames(Sigma) <- var_names
      
      results$`Sigma_u,t` <- round(Sigma, 2)
      
    } else {
      
      Sigma <- compute_stat(posterior$Sigma_u, stat)
      rownames(Sigma) <- var_names
      colnames(Sigma) <- var_names
      
      results$Sigma_u <- round(Sigma, 2)
    }
  }
  
  # ---------------- A ----------------
  if (!is.null(posterior$A)) {
    A <- compute_stat(posterior$A, stat)
    rownames(A) <- var_names
    colnames(A) <- var_names
    results$A <- round(A, 2)
  }
  
  # ---------------- RW SV ----------------
  if (!is.null(posterior$phi)) {
    phi <- apply(posterior$phi, 2, if (stat == "mean") mean else median)
    names(phi) <- var_names
    results$phi <- round(phi, 2)
  }
  
  # ---------------- AR1 SV ----------------
  if (!is.null(posterior$gamma_0)) {
    gamma_0 <- apply(posterior$gamma_0, 2, if (stat == "mean") mean else median)
    names(gamma_0) <- var_names
    results$gamma_0 <- round(gamma_0, 2)
  }
  
  if (!is.null(posterior$gamma_1)) {
    gamma_1 <- apply(posterior$gamma_1, 2, if (stat == "mean") mean else median)
    names(gamma_1) <- var_names
    results$gamma_1 <- round(gamma_1, 2)
  }
  
  if (!is.null(posterior$Phi)) {
    Phi <- compute_stat(posterior$Phi, stat)
    rownames(Phi) <- var_names
    colnames(Phi) <- var_names
    results$Phi <- round(Phi, 2)
  }
  
  # ---------------- filter ----------------
  if (!is.null(pars)) {
    results <- results[names(results) %in% pars]
  }
  
  # ---------------- print ----------------

  cat("Posterior", stat, "estimates\n------------------------\n\n\n")

  
  for (i in seq_along(results)) {
    
    param_name <- names(results)[i]
    obj <- results[[i]]
    
    # -------------------------
    # Sigma_u,t
    # -------------------------
    if (param_name == "Sigma_u,t") {
      
      cat("Sigma_u,t (t = ", t_display, ")\n--------------------------------------------------------------------------------\n", sep = "")
      
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      
    } else if (param_name == "Sigma_u") {
      
      cat("Sigma_u\n--------------------------------------------------------------------------------")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      
    } else if (param_name == "beta") {
      
      cat("beta\n--------------------------------------------------------------------------------")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      
      # -------------------------
      # Psi
      # -------------------------
    } else if (param_name == "Psi") {
      
      cat("Psi\n--------------------------------------------------------------------------------")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      
      # -------------------------
      # A
      # -------------------------
    } else if (param_name == "A") {
      
      cat("A\n--------------------------------------------------------------------------------")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      
      # -------------------------
      # phi (RW SV)
      # -------------------------
    } else if (param_name == "phi") {
      
      cat("phi\n--------------------------------------------------------------------------------\n")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      # -------------------------
      # gamma_0 (AR1 SV)
      # -------------------------
    } else if (param_name == "gamma_0") {
      
      cat("gamma_0\n--------------------------------------------------------------------------------\n")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      # -------------------------
      # gamma_1 (AR1 SV)
      # -------------------------
    } else if (param_name == "gamma_1") {
      
      cat("gamma_1\n--------------------------------------------------------------------------------\n")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      # -------------------------
      # Phi (AR1 SV)
      # -------------------------
    } else if (param_name == "Phi") {
      
      cat("Phi\n--------------------------------------------------------------------------------")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
      # -------------------------
      # fallback (future-proofing)
      # -------------------------
    } else {
      
      cat(param_name, "\n\n")
      print(obj)
      cat("--------------------------------------------------------------------------------\n\n")
    }
    
    if (i < length(results)) cat("\n")
  }
  
  invisible(object)
}

#' Plot stochastic volatility estimates and forecasts
#'
#' Produces time series plots of posterior stochastic volatility estimates
#' over the estimation sample together with predictive paths over the forecast
#' horizon for a fitted steady-state \code{bvar} object with stochastic volatility.
#'
#' @param x A steady-state \code{bvar} object that has been passed through
#'   \code{\link{fit}} with stochastic volatility enabled.
#' @param ci Numeric. Interval level. Default is \code{0.95}.
#' @param vol Character. Volatility representation: \code{"log_lambda"}
#'   (default) or \code{"sd"}.
#' @param stat Character. Posterior summary statistic to use as point estimate:
#'   \code{"mean"} (default) or \code{"median"}.
#' @param plot_idx Integer vector. Indices of variables to plot. If \code{NULL}
#'   (default), all variables are plotted.
#' @param xlim Numeric vector of length 2. Optional x-axis limits.
#' @param ylim Numeric vector of length 2. Optional y-axis limits.
#'
#' @return An invisible list with:
#' \itemize{
#'   \item \code{estimated}: List of posterior summaries for the estimation sample:
#'     \itemize{
#'       \item \code{point}: \eqn{T \times k} matrix of posterior point estimates
#'         (mean or median depending on \code{stat})
#'       \item \code{lower}: \eqn{T \times k} matrix of lower credible bounds
#'       \item \code{upper}: \eqn{T \times k} matrix of upper credible bounds
#'     }
#'   \item \code{predicted}: List of posterior summaries for the forecast horizon:
#'     \itemize{
#'       \item \code{point}: \eqn{(T+H) \times k} matrix of predictive point estimates
#'         (mean or median depending on \code{stat})
#'       \item \code{lower}: \eqn{(T+H) \times k} matrix of lower bound of prediction interval
#'       \item \code{upper}: \eqn{(T+H) \times k} matrix of upper bound of prediction interval
#'     }
#' }
#'
#' @details
#' The function supports two volatility representations:
#' \itemize{
#'   \item \code{"log_lambda"}: log variances
#'   \item \code{"sd"}: implied standard deviations
#' }
#'
#' For each selected variable, the function plots the posterior point estimate
#' path and credible bands over the estimation sample, and the predictive point
#' estimate path and prediction intervals over the forecast horizon.
#'
#' @export
#' @examples
#' \donttest{
#' yt <- matrix(rnorm(50), 25, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p = 1, deterministic = "constant")
#' 
#' k <- bvar_obj$setup$k
#' n_free_params_A <- bvar_obj$setup$n_free_params_A
#' 
#' SV_priors <- list(
#' theta_A             =  rep(0, n_free_params_A),
#' Omega_A             =  diag(1000, n_free_params_A),
#' mu_log_lambda_1     =  rep(0, k),
#' sigma2_log_lambda_1 =  rep(1000, k),
#' alpha_phi           =  rep(5, k),
#' beta_phi            = (rep(5, k) - 1) * rep(0.1, k)
#' )
#'
#' bvar_obj <- priors(bvar_obj,
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    SV = TRUE,
#'                    SV_type = "RW",
#'                    SV_priors = SV_priors)
#'
#' bvar_obj <- fit(bvar_obj,
#'                 H = 8,
#'                 d_pred = matrix(rep(1, 8)),
#'                 iter = 200,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1,
#'                 verbose = FALSE)
#'
#' stochastic_volatility_plot(bvar_obj, ci = 0.95)
#' }
stochastic_volatility_plot <- function(x, ci = 0.95, vol = "log_lambda",
                                       stat = "mean", plot_idx = NULL,
                                       xlim = NULL, ylim = NULL) {
  
  stat     <- match.arg(stat, c("mean", "median"))
  point_fn <- if (stat == "mean") mean else median
  
  draws           <- rstan::extract(x$fit$stan)
  log_lambda_pred <- draws$log_lambda_pred
  Sigma_u_pred    <- draws$Sigma_u_pred
  
  N     <- dim(x$data)[1]
  p     <- x$setup$p
  k     <- x$setup$k
  N_est <- N - p
  H     <- x$predict$H
  alpha <- 1 - ci
  
  if (vol == "sd") {
    
    sigma_post_point <- matrix(NA, N_est, k)
    sigma_post_lower <- matrix(NA, N_est, k)
    sigma_post_upper <- matrix(NA, N_est, k)
    
    for (t in 1:N_est) {
      Sigma_draws_t <- draws$Sigma_u[, t, , ]
      sigma_draws_t <- apply(Sigma_draws_t, 1, function(S) sqrt(diag(S)))
      
      sigma_post_point[t, ] <- apply(sigma_draws_t, 1, point_fn)
      sigma_post_lower[t, ] <- apply(sigma_draws_t, 1, quantile, probs = alpha / 2)
      sigma_post_upper[t, ] <- apply(sigma_draws_t, 1, quantile, probs = 1 - alpha / 2)
    }
    
  } else if (vol == "log_lambda") {
    
    log_lambda_post_point <- apply(draws$log_lambda, c(2, 3), point_fn)
    log_lambda_post_lower <- apply(draws$log_lambda, c(2, 3), quantile, probs = alpha / 2)
    log_lambda_post_upper <- apply(draws$log_lambda, c(2, 3), quantile, probs = 1 - alpha / 2)
    
  } else {
    stop("vol must be either 'log_lambda' or 'sd'")
  }
  
  if (is.null(plot_idx)) plot_idx <- 1:k
  
  user_xlim <- xlim
  user_ylim <- ylim
  
  est_point <- matrix(NA, N_est, k)
  est_lower <- matrix(NA, N_est, k)
  est_upper <- matrix(NA, N_est, k)
  
  pred_point <- matrix(NA, N_est + H, k)
  pred_lower <- matrix(NA, N_est + H, k)
  pred_upper <- matrix(NA, N_est + H, k)
  
  for (i in plot_idx) {
    
    if (vol == "sd") {
      vol_point <- sigma_post_point[, i]
      vol_lower <- sigma_post_lower[, i]
      vol_upper <- sigma_post_upper[, i]
    } else {
      vol_point <- log_lambda_post_point[, i]
      vol_lower <- log_lambda_post_lower[, i]
      vol_upper <- log_lambda_post_upper[, i]
    }
    
    est_point[, i] <- vol_point
    est_lower[, i] <- vol_lower
    est_upper[, i] <- vol_upper
    
    vol_pred_point <- rep(NA, N_est + H)
    vol_pred_lower <- rep(NA, N_est + H)
    vol_pred_upper <- rep(NA, N_est + H)
    
    vol_pred_point[N_est] <- tail(vol_point, 1)
    vol_pred_lower[N_est] <- tail(vol_lower, 1)
    vol_pred_upper[N_est] <- tail(vol_upper, 1)
    
    for (h in 1:H) {
      
      draws_h <- if (vol == "sd") {
        sqrt(Sigma_u_pred[, h, i, i])
      } else {
        log_lambda_pred[, h, i]
      }
      
      vol_pred_point[N_est + h] <- point_fn(draws_h)
      vol_pred_lower[N_est + h] <- quantile(draws_h, alpha / 2)
      vol_pred_upper[N_est + h] <- quantile(draws_h, 1 - alpha / 2)
    }
    
    pred_point[, i] <- vol_pred_point
    pred_lower[, i] <- vol_pred_lower
    pred_upper[, i] <- vol_pred_upper
    
    xlim_i <- if (is.null(user_xlim)) c(0, N + H + 10) else user_xlim
    
    ylim_i <- if (is.null(user_ylim)) {
      range(c(vol_point, vol_lower, vol_upper,
              vol_pred_lower, vol_pred_upper), na.rm = TRUE)
    } else user_ylim
    
    ts.plot(vol_point,
            col  = "red", lwd = 2,
            main = if (vol == "sd") paste0("sd(u_", i, ")")
            else paste0("ln(lambda_", i, ")"),
            ylim = ylim_i, xlim = xlim_i, ylab = "")
    
    x_insample <- 1:N_est
    polygon(x = c(x_insample, rev(x_insample)),
            y = c(vol_upper, rev(vol_lower)),
            col = rgb(1, 0, 0, 0.15), border = NA)
    
    x_fore <- N_est:(N_est + H)
    polygon(x = c(x_fore, rev(x_fore)),
            y = c(vol_pred_upper[x_fore], rev(vol_pred_lower[x_fore])),
            col = rgb(0, 0, 1, 0.2), border = NA)
    
    lines(x_fore, vol_pred_point[x_fore], col = "blue", lwd = 2)
    
    legend("topleft",
           legend = c("Estimated volatility", "Predicted volatility"),
           col    = c("red", "blue"),
           lwd    = 2,
           bty    = "n")
  }
  
  invisible(list(
    estimated = list(
      point = est_point,
      lower = est_lower,
      upper = est_upper
    ),
    predicted = list(
      point = pred_point,
      lower = pred_lower,
      upper = pred_upper
    )
  ))
}

#' Specify priors for the steady-state BVAR model
#'
#' This function prepares the priors. The Minnesota prior is used for the autoregressive parameters, and is determined by
#' the overall tightness, cross-equation tightness, and the lag decay rate.
#' For the steady-state parameters, a normal prior is used. For the covariance matrix of the innovations,
#' the user can choose between Jeffreys prior or an uninformative inverse-Wishart prior.
#' Optionally enables stochastic volatility where the covariance matrix varies over time.
#'
#' @param x A steady-state \code{bvar} object that has been passed through \code{\link{setup}}.
#' @param lambda_1 Numeric. Overall tightness of the Minnesota prior.
#'   Default \code{0.2}.
#' @param lambda_2 Numeric. Cross-equation tightness of the Minnesota prior. Default \code{0.5}.
#' @param lambda_3 Numeric. Lag decay rate of the Minnesota prior. Default \code{1}.
#' @param first_own_lag_prior_mean Numeric vector of length \code{k}. Prior means for the first own lags
#'   of the variables. If \code{NULL} (default), a zero vector is used.
#' @param theta_Psi Numeric vector. Prior mean vector for \eqn{\text{vec}(\Psi)}, i.e. the steady-state parameters. If \code{NULL} (default),
#'   the OLS estimates are used.
#' @param Omega_Psi Numeric matrix. Prior covariance matrix for \eqn{\text{vec}(\Psi)}, i.e. the steady-state parameters. If \code{NULL} (default),
#'    a diagonal matrix with variances \code{1000} is used.
#' @param Jeffreys Logical. If \code{TRUE} (default), uses Jeffreys prior for the innovation covariance matrix.
#'   If \code{FALSE}, uses an uninformative inverse-Wishart prior. Only considered if \code{SV=FALSE}.
#' @param SV Logical. If \code{TRUE}, enables stochastic volatility specification.
#'   Default \code{FALSE}.
#' @param SV_type Character. Type of stochastic volatility model. Must be either \code{"RW"} or \code{"AR1"}.
#'   Required if \code{SV = TRUE}.
#' @param SV_priors List. User-supplied stochastic volatility priors. Required when \code{SV = TRUE}.
#'   The list must contain the following named elements depending on \code{SV_type}:
#'   \itemize{
#'     \item For \code{"RW"}: \code{theta_A}, \code{Omega_A}, \code{mu_log_lambda_1},
#'       \code{sigma2_log_lambda_1}, \code{alpha_phi}, \code{beta_phi}.
#'     \item For \code{"AR1"}: \code{theta_A}, \code{Omega_A}, \code{theta_gamma_0},
#'       \code{Omega_gamma_0}, \code{theta_gamma_1}, \code{Omega_gamma_1},
#'       \code{theta_log_lambda_1}, \code{Omega_log_lambda_1}, \code{V_Phi}, \code{m_Phi}.
#'   }
#'
#' @return The steady-state \code{bvar} object with an appended \code{priors} list containing:
#'   \item{theta_beta}{Prior mean vector for \eqn{\text{vec}(\beta)} constructed with the Minnesota prior}
#'   \item{Omega_beta}{Prior covariance matrix for \eqn{\text{vec}(\beta)} constructed with the Minnesota prior}
#'   \item{theta_Psi}{Prior mean vector for \eqn{\text{vec}(\Psi)}, i.e. the steady-state parameters}
#'   \item{Omega_Psi}{Prior covariance matrix for \eqn{\text{vec}(\Psi)}, i.e. the steady-state parameters}
#'   \item{Jeffreys}{Indicator for Jeffreys prior usage}
#'   \item{Sigma_AR}{Residual variance estimates from univariate AR fits, which are used by the Minnesota prior}
#'   \item{m}{Inverse-Wishart prior degrees of freedom (if \code{Jeffreys = FALSE})}
#'   \item{V}{Inverse-Wishart prior scale matrix (if \code{Jeffreys = FALSE})}
#'   \item{SV}{Logical indicator for stochastic volatility specification}
#'   \item{SV_type}{Stochastic volatility specification type}
#'   \item{SV_priors}{User-supplied SV prior list (if \code{SV = TRUE})}
#'   
#' @details
#' 
#' The goal is to estimate \eqn{\beta, \Psi}, and \eqn{\Sigma_u}, so priors are needed.
#' Following Villani (2009), prior independence between \eqn{\beta, \Psi} and \eqn{\Sigma_u} is assumed. For \eqn{\beta}, i.e. the autoregressive parameter matrix,
#' the Minnesota prior is used
#'
#' \deqn{\mathrm{vec}(\beta) \sim \mathrm{N}_{kpk} \left[\theta_\beta,\Omega_\beta\right]}
#'
#' The prior means (the elements of \eqn{\theta_\beta}) are set to
#'
#' \deqn{
#' \begin{aligned}
#' \mathrm{E}\left(\Pi_{\ell}^{(i,j)}\right)&=
#' \begin{cases}\kappa & \text{if } \ell = 1 \ \text{and} \ i = j \\0 & \text{otherwise}\end{cases}\\
#' \kappa&=\begin{cases}\kappa^{level} & \text{if } \text{variable} \ i \ \text{is in level} \\
#' \kappa^{\Delta} & \text{if } \text{variable} \ i \ \text{is in difference}
#' \end{cases}\\
#' \end{aligned}
#' }
#'
#' Here, the autoregressive coefficient \eqn{\Pi_{\ell}^{(i,j)}} is element \eqn{\left(i,j\right)}
#' of \eqn{\Pi_{\ell}} for \eqn{\ell=1,\dots,p}. As such, the Minnesota prior sets all prior means
#' for the elements in \eqn{\beta} to \eqn{0}, except for the elements that relate to the first
#' own lags of the variables, which are set to \eqn{\kappa}. If variable \eqn{i} is in level
#' (e.g. nominal interest rate), then \eqn{\kappa=\kappa^{level}}, and typical choices for
#' \eqn{\kappa^{level}} are \eqn{1} or \eqn{0.9}. Evaluating the equations at their prior means,
#' equation \eqn{i} becomes a random walk if \eqn{\kappa^{level}=1} and a persistent stationary
#' AR(1) process if \eqn{\kappa^{level}=0.9}. Since the steady state only exists if the process
#' is stationary, \eqn{0.9} is recommended for the steady-state BVAR. If variable \eqn{i} is in difference
#' (e.g. output growth), then \eqn{\kappa=\kappa^{\Delta}}, and the most common choice for
#' \eqn{\kappa^{\Delta}} is \eqn{0}, i.e. equation \eqn{i} becomes (when evaluating it at its prior means)
#' a random walk expressed in first differences. If a differenced variable still shows some degree
#' of persistence (can be examined with an ACF plot), a suitable value for \eqn{\kappa^{\Delta}} can
#' be (for example) \eqn{0.6} instead of \eqn{0}. Moving on to the prior variances, \eqn{\Omega_\beta} is a
#' diagonal matrix containing the prior variances for the elements in \eqn{\beta}. They are specified as
#' 
#' \deqn{\mathrm{Var}\left(\Pi_{\ell}^{(i,j)}\right)=
#' \begin{cases}\left(\frac{\lambda_1}{\ell^{\lambda_3}}\right)^2 & \text{if } i = j \\
#' \left(\frac{\lambda_1 \lambda_2\sigma_i}{\ell^{\lambda_3}\sigma_j}\right)^2& \text{if } i \neq j
#' \end{cases}}
#' 
#' Here \eqn{\lambda_1}, \eqn{\lambda_2}, and \eqn{\lambda_3} are scalar hyperparameters known as
#' the overall tightness, the cross-equation tightness and the lag decay rate. Furthermore,
#' \eqn{\sigma_i^2} is the \eqn{(i,i)}:th element of \eqn{\Sigma_u}, which is unknown and therefore
#' replaced with an estimate. In this package, it is replaced by the least squares residual variance
#' from a univariate autoregression for variable \eqn{i} with \eqn{p} lags
#' (including the constant and dummy/trend variable if applicable). Moving on to \eqn{\Psi}, the steady-state parameter matrix, the
#' prior is
#' 
#' \deqn{\mathrm{vec}(\Psi) \sim \mathrm{N}_{kq}\left[\theta_\Psi,\Omega_\Psi\right]}
#' 
#' This is the core of the steady-state BVAR model.
#' In \eqn{\theta_\Psi}, one specifies the prior beliefs about the location of the steady state,
#' and in \eqn{\Omega_\Psi}, which is assumed to be a diagonal matrix, one specifies the degree
#' of certainty in those prior beliefs. The prior for \eqn{\Sigma_u} is either the usual non-informative Jeffreys prior
#' 
#' \deqn{p(\Sigma_u) \propto\left|\Sigma_u \right|^{-(k+1)/2}}
#' 
#' or a proper uninformative inverse-Wishart prior
#' 
#' \deqn{\Sigma_u \sim \mathrm{IW}(V,m)}
#' 
#' where \eqn{V} is the scale matrix and \eqn{m} is the number of degrees of freedom.
#' An uninformative prior is specified by setting
#' \eqn{V=(m-k-1)\hat{\Sigma}_u} where \eqn{\hat{\Sigma}_u} is the least squares estimate
#' from the VAR(\eqn{p}) (including the constant and dummy/trend variable if applicable), and \eqn{m=k+2}.
#' For the stochastic volatility specifications, the innovation covariance matrix is now time-varying \eqn{\Sigma_{u,t}}.
#' Therefore, stochastic volatility priors are needed, see \link{bvar} for more details. For the Random Walk
#' (\code{"RW"}) stochastic volatility specification, the following priors are used
#' 
#' \deqn{\begin{aligned}a &\sim \mathrm{N}(\theta_A, \Omega_A) \\
#' \ln \lambda_{i,1} &\sim \mathrm{N}(\mu_{\ln \lambda_{i,1}}, \sigma^2_{\ln \lambda_{i,1}}) \\
#' \phi_i &\sim \mathrm{IG}(\alpha_{\phi_i},\beta_{\phi_i})\end{aligned}}
#' 
#' Here \eqn{a} is a \eqn{k(k-1)/2}-dimensional vector that collects the free parameters in \eqn{A} in row-major order,
#' and \eqn{\ln \lambda_{i,1}} are the time \eqn{t=1} values (initial conditions) of \eqn{\ln \lambda_{i,t}} for \eqn{i=1,\dots,k}.
#' Furthermore, \eqn{\phi_i} for \eqn{i=1,\dots,k} are the log volatility innovation variances. For the AR(1) (\code{"AR1"}) stochastic volatility specification, the following priors are used
#' 
#' \deqn{\begin{aligned}a &\sim \mathrm{N}(\theta_A, \Omega_A) \\
#' \gamma_{0} &\sim \mathrm{N}(\theta_{\gamma_0}, \Omega_{\gamma_0}) \\
#' \gamma_{1} &\sim \mathrm{N}(\theta_{\gamma_1}, \Omega_{\gamma_1}) \\
#' \ln \lambda_{1} &\sim \mathrm{N}(\theta_{\ln \lambda_{1}}, \Omega_{\ln \lambda_{1}}) \\
#' \Phi &\sim \mathrm{IW}(V_{\Phi},m_{\Phi})\end{aligned}}
#' 
#' Here \eqn{a} is again the \eqn{k(k-1)/2}-dimensional vector that collects the free parameters in \eqn{A} in row-major order,
#' and \eqn{\ln \lambda_1} is a \eqn{k}-dimensional vector containing the time \eqn{t=1} values (initial conditions) of \eqn{\ln \lambda_{t}}.
#' Furthermore, \eqn{\gamma_{0}} is a \eqn{k}-dimensional vector of log volatility intercepts, \eqn{\gamma_{1}} is a \eqn{k}-dimensional vector of log volatility
#' slopes, and \eqn{\Phi} is the \eqn{k \times k} log volatility innovation covariance matrix.
#' 
#' For details on the homoscedastic steady-state BVAR model, see Villani (2009).
#' For details on the Random Walk stochastic volatility steady-state BVAR model, see Clark (2011).
#' See Carriero, Clark, and Marcellino (2024) for the AR(1) stochastic volatility
#' specification applied to a conventional BVAR.
#' 
#' @references
#' Carriero, A., Clark, T. E., and Marcellino, M. (2024).
#' Capturing macro-economic tail risks with Bayesian vector autoregressions. 
#' \emph{Journal of Money, Credit and Banking}, 56(5), pp. 1099–1127.
#' 
#' Clark, T. E. (2011). Real-time density forecasts from Bayesian vector autoregressions
#' with stochastic volatility. \emph{Journal of Business & Economic Statistics}, 29(3), pp. 327–341.
#' 
#' Villani, M. (2009). Steady-state priors for vector autoregressions.
#' \emph{Journal of Applied Econometrics}, 24(4), pp. 630–650. 
#' 
#' @export
#'
#' @examples
#' #homoscedastic with Jeffreys prior
#' yt <- matrix(rnorm(50), 25, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=1)
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    Jeffreys = TRUE,
#'                    SV = FALSE,
#'                    SV_type = NULL,
#'                    SV_priors = NULL)
#'                    
#' #RW stochastic volatility
#' yt <- matrix(rnorm(50), 25, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=1)
#' 
#' k <- bvar_obj$setup$k
#' n_free_params_A <- bvar_obj$setup$n_free_params_A
#' 
#' SV_priors_RW <- list(
#' theta_A              =  rep(0, n_free_params_A),
#' Omega_A              =  diag(1000, n_free_params_A),
#' mu_log_lambda_1      =  rep(0, k),
#' sigma2_log_lambda_1  =  rep(1000, k),
#' alpha_phi            =  rep(5, k),
#' beta_phi             = (rep(5, k) - 1) * rep(0.1, k)
#' )
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    SV = TRUE,
#'                    SV_type = "RW",
#'                    SV_priors = SV_priors_RW)
#'                    
#' #AR1 stochastic volatility
#' yt <- matrix(rnorm(50), 25, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=1)
#' 
#' k <- bvar_obj$setup$k
#' n_free_params_A <- bvar_obj$setup$n_free_params_A
#' 
#' SV_priors_AR1 <- list(
#' theta_A               =  rep(0, n_free_params_A),
#' Omega_A               =  diag(1000, n_free_params_A),
#' theta_gamma_0         =  rep(0.1, k),
#' Omega_gamma_0         =  diag(1000, k),
#' theta_gamma_1         =  rep(0.9, k),
#' Omega_gamma_1         =  diag(10, k),
#' theta_log_lambda_1    =  rep(0.1, k)/(1-rep(0.9, k)),
#' Omega_log_lambda_1    =  diag(1000, k),
#' V_Phi                 = (10 - k - 1) * diag(k),
#' m_Phi                 =  10
#' )
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    SV = TRUE,
#'                    SV_type = "AR1",
#'                    SV_priors = SV_priors_AR1)
#'
priors<- function(x,
                  lambda_1=0.2,
                  lambda_2=0.5,
                  lambda_3 = 1,
                  first_own_lag_prior_mean=NULL,
                  theta_Psi=NULL,
                  Omega_Psi=NULL,
                  Jeffreys=TRUE,
                  SV = FALSE,
                  SV_type = NULL,
                  SV_priors = NULL){
  
  if (!inherits(x, "bvar")) stop("must be a 'bvar' object")
  if (is.null(x$setup)) stop("must be passed through setup")
  
  setup <- x$setup
  
  if (!all(c("k","p","q","dt") %in% names(setup))) {
    stop("setup is incomplete")
  }
  
  k <- setup$k
  p <- setup$p
  q <- setup$q
  dt <- setup$dt
  yt <- x$data
  
  if (!is.null(theta_Psi) && !is.null(Omega_Psi)) {
    if (nrow(Omega_Psi) != length(theta_Psi)) {
      stop("dimensions must match")
    }
  }
  
  if (any(c(lambda_1, lambda_2, lambda_3) <= 0)) {
    stop("must be positive")
  }
  
  if (isTRUE(SV)) {
    if (is.null(SV_type)) stop("SV_type is needed")
    if (!SV_type %in% c("RW", "AR1")) stop("SV_type needs to be RW or AR1")
    if (is.null(SV_priors)) stop("SV_priors is required")
    
    n_free <- x$setup$n_free_params_A
    
    if (SV_type == "RW") {
      required_names <- c("theta_A", "Omega_A", "mu_log_lambda_1",
                          "sigma2_log_lambda_1", "alpha_phi", "beta_phi")
      missing_names <- setdiff(required_names, names(SV_priors))
      if (length(missing_names) > 0)
        stop(paste("SV_priors is missing elements:", paste(missing_names, collapse = ", ")))
      
      if (length(SV_priors$theta_A) != n_free)
        stop(paste("theta_A must be a vector of length k*(k-1)/2 =", n_free))
      if (!all(dim(SV_priors$Omega_A) == n_free))
        stop(paste("Omega_A must be a", n_free, "x", n_free, "matrix"))
      if (length(SV_priors$mu_log_lambda_1) != k)
        stop(paste("mu_log_lambda_1 must be a vector of length k =", k))
      if (length(SV_priors$sigma2_log_lambda_1) != k)
        stop(paste("sigma2_log_lambda_1 must be a vector of length k =", k))
      if (any(SV_priors$sigma2_log_lambda_1 <= 0))
        stop("sigma2_log_lambda_1 must be strictly positive")
      if (length(SV_priors$alpha_phi) != k)
        stop(paste("alpha_phi must be a vector of length k =", k))
      if (any(SV_priors$alpha_phi <= 0))
        stop("alpha_phi must be strictly positive")
      if (length(SV_priors$beta_phi) != k)
        stop(paste("beta_phi must be a vector of length k =", k))
      if (any(SV_priors$beta_phi <= 0))
        stop("beta_phi must be strictly positive")
      
    } else if (SV_type == "AR1") {
      required_names <- c("theta_A", "Omega_A", "theta_gamma_0", "Omega_gamma_0",
                          "theta_gamma_1", "Omega_gamma_1", "theta_log_lambda_1",
                          "Omega_log_lambda_1", "m_Phi", "V_Phi")
      missing_names <- setdiff(required_names, names(SV_priors))
      if (length(missing_names) > 0)
        stop(paste("SV_priors is missing elements:", paste(missing_names, collapse = ", ")))
      
      if (length(SV_priors$theta_A) != n_free)
        stop(paste("theta_A must be a vector of length k*(k-1)/2 =", n_free))
      if (!all(dim(SV_priors$Omega_A) == n_free))
        stop(paste("Omega_A must be a", n_free, "x", n_free, "matrix"))
      if (length(SV_priors$theta_gamma_0) != k)
        stop(paste("theta_gamma_0 must be a vector of length k =", k))
      if (!all(dim(SV_priors$Omega_gamma_0) == k))
        stop(paste("Omega_gamma_0 must be a", k, "x", k, "matrix"))
      if (length(SV_priors$theta_gamma_1) != k)
        stop(paste("theta_gamma_1 must be a vector of length k =", k))
      if (!all(dim(SV_priors$Omega_gamma_1) == k))
        stop(paste("Omega_gamma_1 must be a", k, "x", k, "matrix"))
      if (length(SV_priors$theta_log_lambda_1) != k)
        stop(paste("theta_log_lambda_1 must be a vector of length k =", k))
      if (!all(dim(SV_priors$Omega_log_lambda_1) == k))
        stop(paste("Omega_log_lambda_1 must be a", k, "x", k, "matrix"))
      if (!is.numeric(SV_priors$m_Phi) || length(SV_priors$m_Phi) != 1 || SV_priors$m_Phi < k)
        stop(paste("m_Phi must be a scalar integer >= k =", k))
      if (!all(dim(SV_priors$V_Phi) == k))
        stop(paste("V_Phi must be a", k, "x", k, "matrix"))
    }
  }
  
  priors <- list()
  
  Sigma_AR <- x$setup$Sigma_AR
  sigma <- sqrt(diag(Sigma_AR))
  
  Vv <- lapply(1:p, function(x) matrix(0, k, k))
  
  for (l in 1:p) {
    for (i in 1:k) {
      for (j in 1:k) {
        if (i == j) {
          Vv[[l]][i,j] <- (lambda_1/(l^lambda_3))^2
        } else {
          Vv[[l]][i,j] <- ((lambda_1*lambda_2*sigma[i])/(l^lambda_3*sigma[j]))^2
        }
      }
    }
  }
  
  V_mat <- do.call(cbind, Vv)
  Omega_beta <- diag(c(t(V_mat)))
  
  if (!is.null(first_own_lag_prior_mean)) {
    if (length(first_own_lag_prior_mean) != k) {
      stop("must have length k")
    }
  }
  
  if (is.null(first_own_lag_prior_mean)) first_own_lag_prior_mean <- rep(0,k)
  
  mat <- matrix(0, nrow = k*p, ncol = k)
  for (i in 1:k){
    mat[i,i] <- first_own_lag_prior_mean[i]
  }
  theta_beta = c(mat)
  
  if (is.null(theta_Psi)) theta_Psi <- c(x$setup$Psi_OLS)
  if (is.null(Omega_Psi)) Omega_Psi <- diag(1000, k*q, k*q)
  
  if (isFALSE(Jeffreys)){
    m=k+2
    V = (m-k-1)*setup$Sigma_u_OLS
    priors$V <- V
    priors$m <- m
  }
  
  priors$theta_beta <- theta_beta
  priors$Omega_beta <- Omega_beta
  priors$theta_Psi <- theta_Psi
  priors$Omega_Psi <- Omega_Psi
  priors$Jeffreys <- Jeffreys
  priors$Sigma_AR <- Sigma_AR
  
  priors$SV <- SV
  priors$SV_type <- SV_type
  
  if (isTRUE(SV)) {
    priors$SV_priors <- SV_priors
  }
  
  x$priors <- priors
  return(x)
}

test_that("restrict_beta stores restriction matrix", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data)
  model <- SteadyStateBVAR::setup(model, p = 2)
  model <- priors(model)
  
  k <- model$setup$k
  p <- model$setup$p
  R <- matrix(1, nrow = k * p, ncol = k)
  R[2, 1] <- 0
  
  result <- restrict_beta(model, R)
  
  expect_equal(result$setup$restriction_matrix, R)
})

test_that("restrict_beta updates Omega_beta for zero restrictions", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data)
  model <- SteadyStateBVAR::setup(model, p = 2)
  model <- priors(model)
  
  k <- model$setup$k
  p <- model$setup$p
  R <- matrix(1, nrow = k * p, ncol = k)
  R[2, 1] <- 0
  
  zero_idx <- which(c(R) == 0)
  result <- restrict_beta(model, R)
  
  expect_equal(diag(result$priors$Omega_beta)[zero_idx], 1e-7)
})

test_that("restrict_beta checks matrix dimensions", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data)
  model <- SteadyStateBVAR::setup(model, p = 2)
  model <- priors(model)
  
  k <- model$setup$k
  p <- model$setup$p
  R_bad <- matrix(1, nrow = k * p - 1, ncol = k)
  
  expect_error(
    restrict_beta(model, R_bad),
    "restriction_matrix must have dimension"
  )
})

test_that("restrict_beta warns if Omega_beta is missing", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data)
  model <- SteadyStateBVAR::setup(model, p = 2)
  model$priors <- list()
  
  k <- model$setup$k
  p <- model$setup$p
  R <- matrix(1, nrow = k * p, ncol = k)
  
  expect_warning(
    restrict_beta(model, R),
    "Omega_beta not found"
  )
})

test_that("fit requires bvar object", {
  expect_error(fit(list()), "must be a 'bvar' object")
})

test_that("fit requires setup", {
  model <- bvar(data = matrix(rnorm(300), nrow = 100, ncol = 3))
  expect_error(fit(model), "must be passed through setup")
})

test_that("fit requires priors", {
  model <- bvar(data = matrix(rnorm(300), nrow = 100, ncol = 3))
  model <- SteadyStateBVAR::setup(model, p = 2)
  expect_error(fit(model), "must be passed through priors")
})

test_that("fit requires d_pred", {
  model <- bvar(data = matrix(rnorm(300), nrow = 100, ncol = 3))
  model <- SteadyStateBVAR::setup(model, p = 2)
  model <- priors(model)
  
  expect_error(
    fit(model),
    "d_pred must be supplied"
  )
})

test_that("fit validates H", {
  model <- bvar(data = matrix(rnorm(300), nrow = 100, ncol = 3))
  model <- SteadyStateBVAR::setup(model, p = 2)
  model <- priors(model)
  
  expect_error(
    fit(model, H = 0, d_pred = matrix(0, 1, 1)),
    "H must be a positive integer"
  )
})

test_that("fit validates d_pred type", {
  model <- bvar(data = matrix(rnorm(300), nrow = 100, ncol = 3))
  model <- SteadyStateBVAR::setup(model, p = 2)
  model <- priors(model)
  
  expect_error(
    fit(model, H = 1, d_pred = 1),
    "d_pred must be a matrix"
  )
})

test_that("fit validates d_pred row dimension", {
  model <- bvar(data = matrix(rnorm(300), nrow = 100, ncol = 3))
  model <- SteadyStateBVAR::setup(model, p = 2)
  model <- priors(model)
  
  expect_error(
    fit(model, H = 2, d_pred = matrix(0, 1, 3)),
    "nrow\\(d_pred\\) must equal H"
  )
})

test_that("fit validates d_pred column dimension", {
  model <- bvar(data = matrix(rnorm(300), nrow = 100, ncol = 3))
  model <- SteadyStateBVAR::setup(model, p = 2)
  model <- priors(model)
  
  expect_error(
    fit(model, H = 1, d_pred = matrix(0, 1, 999)),
    "ncol\\(d_pred\\) must equal q"
  )
})

test_that("priors requires bvar object", {
  expect_error(priors(list()), "must be a 'bvar' object")
})

test_that("priors requires setup to be run first", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  
  expect_error(priors(model), "must be passed through setup")
})

test_that("priors validates lambda parameters are positive", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  expect_error(priors(model, lambda_1 = -0.1), "must be positive")
  expect_error(priors(model, lambda_2 = 0), "must be positive")
  expect_error(priors(model, lambda_3 = -1), "must be positive")
})

test_that("priors validates first_own_lag_prior_mean length", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  expect_error(
    priors(model, first_own_lag_prior_mean = c(1,2)),
    "must have length k"
  )
})

test_that("priors validates theta_Psi and Omega_Psi dimensions match", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  expect_error(
    priors(model, theta_Psi = c(1,2,3), Omega_Psi = diag(2)),
    "dimensions must match"
  )
})

test_that("priors default behavior", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  result <- priors(model)
  
  expect_s3_class(result, "bvar")
  expect_true(!is.null(result$priors))
})

test_that("theta_beta structure correct", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  result <- priors(model)
  
  expect_equal(length(result$priors$theta_beta), 18)
})

test_that("Omega_beta structure correct", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  result <- priors(model)
  
  expect_equal(dim(result$priors$Omega_beta), c(18, 18))
})

test_that("Sigma_AR computed", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  result <- priors(model)
  
  expect_equal(dim(result$priors$Sigma_AR), c(3,3))
})

test_that("Jeffreys FALSE adds hyperparameters", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  r1 <- priors(model, Jeffreys = TRUE)
  r2 <- priors(model, Jeffreys = FALSE)
  
  expect_null(r1$priors$m)
  expect_false(is.null(r2$priors$m))
})

test_that("Jeffreys FALSE hyperparameters have expected values", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  result <- priors(model, Jeffreys = FALSE)
  
  expect_equal(result$priors$m, k + 2)
  expect_equal(result$priors$V, (result$priors$m - k - 1) * model$setup$Sigma_u_OLS)
})

test_that("SV requires SV_priors", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "AR1"),
    "SV_priors is required"
  )
})

test_that("RW SV_priors validates required names", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A = rep(0, n_free),
    Omega_A = diag(n_free)
    # missing: mu_log_lambda_1, sigma2_log_lambda_1, alpha_phi, beta_phi
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "RW", SV_priors = sv),
    "SV_priors is missing elements"
  )
})

test_that("RW SV_priors validates theta_A length", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A             = rep(0, n_free + 1), # wrong length
    Omega_A             = diag(n_free),
    mu_log_lambda_1     = rep(0, k),
    sigma2_log_lambda_1 = rep(1, k),
    alpha_phi           = rep(1, k),
    beta_phi            = rep(1, k)
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "RW", SV_priors = sv),
    "theta_A must be a vector of length"
  )
})

test_that("RW SV_priors validates Omega_A dimensions", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A             = rep(0, n_free),
    Omega_A             = diag(n_free + 1), # wrong dimensions
    mu_log_lambda_1     = rep(0, k),
    sigma2_log_lambda_1 = rep(1, k),
    alpha_phi           = rep(1, k),
    beta_phi            = rep(1, k)
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "RW", SV_priors = sv),
    "Omega_A must be a"
  )
})

test_that("RW SV_priors validates sigma2_log_lambda_1 positivity", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A             = rep(0, n_free),
    Omega_A             = diag(n_free),
    mu_log_lambda_1     = rep(0, k),
    sigma2_log_lambda_1 = rep(-1, k), # must be positive
    alpha_phi           = rep(1, k),
    beta_phi            = rep(1, k)
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "RW", SV_priors = sv),
    "sigma2_log_lambda_1 must be strictly positive"
  )
})

test_that("RW SV_priors validates alpha_phi positivity", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A             = rep(0, n_free),
    Omega_A             = diag(n_free),
    mu_log_lambda_1     = rep(0, k),
    sigma2_log_lambda_1 = rep(1, k),
    alpha_phi           = rep(-1, k), # must be positive
    beta_phi            = rep(1, k)
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "RW", SV_priors = sv),
    "alpha_phi must be strictly positive"
  )
})

test_that("RW SV_priors validates beta_phi positivity", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A             = rep(0, n_free),
    Omega_A             = diag(n_free),
    mu_log_lambda_1     = rep(0, k),
    sigma2_log_lambda_1 = rep(1, k),
    alpha_phi           = rep(1, k),
    beta_phi            = rep(-1, k) # must be positive
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "RW", SV_priors = sv),
    "beta_phi must be strictly positive"
  )
})

test_that("RW SV attaches correctly with valid priors", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A             = rep(0, n_free),
    Omega_A             = diag(n_free),
    mu_log_lambda_1     = rep(0, k),
    sigma2_log_lambda_1 = rep(1, k),
    alpha_phi           = rep(5, k),
    beta_phi            = rep(0.4, k)
  )
  
  result <- priors(model, SV = TRUE, SV_type = "RW", SV_priors = sv)
  
  expect_true(result$priors$SV)
  expect_equal(result$priors$SV_type, "RW")
  expect_equal(result$priors$SV_priors, sv)
})

test_that("AR1 SV_priors validates required names", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A = rep(0, n_free),
    Omega_A = diag(n_free)
    # missing remaining AR1 elements, including V_Phi and m_Phi
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "AR1", SV_priors = sv),
    "SV_priors is missing elements"
  )
})

test_that("AR1 SV_priors validates Omega_gamma_0 dimensions", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A            = rep(0, n_free),
    Omega_A            = diag(n_free),
    theta_gamma_0      = rep(0, k),
    Omega_gamma_0      = diag(k + 1), # wrong dimensions
    theta_gamma_1      = rep(0, k),
    Omega_gamma_1      = diag(k),
    theta_log_lambda_1 = rep(0, k),
    Omega_log_lambda_1 = diag(k),
    m_Phi              = k + 2,
    V_Phi              = diag(k)
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "AR1", SV_priors = sv),
    "Omega_gamma_0 must be a"
  )
})

test_that("AR1 SV_priors validates m_Phi >= k", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A            = rep(0, n_free),
    Omega_A            = diag(n_free),
    theta_gamma_0      = rep(0, k),
    Omega_gamma_0      = diag(k),
    theta_gamma_1      = rep(0, k),
    Omega_gamma_1      = diag(k),
    theta_log_lambda_1 = rep(0, k),
    Omega_log_lambda_1 = diag(k),
    m_Phi              = k - 1, # too small
    V_Phi              = diag(k)
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "AR1", SV_priors = sv),
    "m_Phi must be a scalar integer >= k"
  )
})

test_that("AR1 SV_priors validates V_Phi dimensions", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A            = rep(0, n_free),
    Omega_A            = diag(n_free),
    theta_gamma_0      = rep(0, k),
    Omega_gamma_0      = diag(k),
    theta_gamma_1      = rep(0, k),
    Omega_gamma_1      = diag(k),
    theta_log_lambda_1 = rep(0, k),
    Omega_log_lambda_1 = diag(k),
    m_Phi              = k + 2,
    V_Phi              = diag(k + 1) # wrong dimensions
  )
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "AR1", SV_priors = sv),
    "V_Phi must be a"
  )
})

test_that("AR1 SV attaches correctly with valid priors", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  k <- model$setup$k
  n_free <- model$setup$n_free_params_A
  
  sv <- list(
    theta_A            = rep(0, n_free),
    Omega_A            = diag(n_free),
    theta_gamma_0      = rep(0, k),
    Omega_gamma_0      = diag(k),
    theta_gamma_1      = rep(0.9, k),
    Omega_gamma_1      = diag(k),
    theta_log_lambda_1 = rep(0, k),
    Omega_log_lambda_1 = diag(k),
    m_Phi              = k + 2,
    V_Phi              = diag(k)
  )
  
  result <- priors(model, SV = TRUE, SV_type = "AR1", SV_priors = sv)
  
  expect_true(result$priors$SV)
  expect_equal(result$priors$SV_type, "AR1")
  expect_equal(result$priors$SV_priors, sv)
})

test_that("SV_type must be RW or AR1", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  model <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  expect_error(
    priors(model, SV = TRUE, SV_type = "invalid", SV_priors = list()),
    "SV_type needs to be RW or AR1"
  )
})

#' Restrict VAR coefficients to zero
#'
#' Applies zero restrictions to the VAR coefficient matrix \eqn{\beta} by setting the
#' corresponding prior variances in \code{Omega_beta} to a value near zero (prior means are zero by default in the Minnesota prior).
#' 
#' @details
#' The steady-state BVAR model takes the form
#'
#' \deqn{y_t = \Psi d_t + \Pi_1(y_{t-1}-\Psi d_{t-1})+\dots+\Pi_p(y_{t-p}-\Psi d_{t-p})+u_t}
#'
#' where \eqn{y_t} is an \eqn{k}-dimensional vector of endogenous variables at time \eqn{t}, and \eqn{d_t} is
#' a \eqn{q}-dimensional vector of deterministic (exogenous) variables at time \eqn{t}.
#' Here \eqn{\Pi_\ell} for \eqn{\ell=1,\dots,p} is a \eqn{(k \times k)} matrix of autoregressive parameters,
#' and \eqn{\Psi} is a \eqn{(k \times q)} matrix of steady-state parameters.
#' One can stack the (transposed) \eqn{\Pi_i} matrices in the \eqn{(kp \times k)} matrix \eqn{\beta}
#' \deqn{\beta=\begin{bmatrix}\Pi'_1 \\ \vdots  \\\Pi'_p\end{bmatrix}}
#' 
#' This function puts zero restrictions on the elements of \eqn{\beta}.
#'
#' @param x A steady-state \code{bvar} object that has been passed through \code{\link{priors}}.
#' @param restriction_matrix A numeric matrix of dimension \code{k*p x k} where
#'   entries of \code{0} indicate coefficients to be restricted to zero and
#'   entries of \code{1} indicate unrestricted coefficients.
#'
#' @return The steady-state \code{bvar} object with the restriction matrix stored in
#'   \code{setup} and the prior covariance matrix \code{Omega_beta} updated accordingly.
#' @export
#'
#' @examples
#' yt <- matrix(rnorm(50), 25, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=2)
#'
#' bvar_obj <- priors(bvar_obj,
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2))
#'                    
#' p <- bvar_obj$setup$p
#' k <- bvar_obj$setup$k
#' 
#' restriction_matrix <- matrix(1,k*p,k, dimnames=list(NULL,c("y1","y2")))
#' 
#' #restrict beta so y2 does not granger-cause y1
#' 
#' restriction_matrix[2, 1] <- 0
#' restriction_matrix[4, 1] <- 0
#' 
#' print(restriction_matrix)
#' 
#' bvar_obj <- restrict_beta(bvar_obj, restriction_matrix)
restrict_beta <- function(x, restriction_matrix) {
  
  k <- x$setup$k
  p <- x$setup$p
  
  if (!all(dim(restriction_matrix) == c(k * p, k))) {
    stop("restriction_matrix must have dimension (k*p x k)")
  }
  
  x$setup$restriction_matrix <- restriction_matrix
  zero_indices <- which(c(restriction_matrix) == 0)
  
  if (!is.null(x$priors$Omega_beta)) {
    diag(x$priors$Omega_beta)[zero_indices] <- 0.0000001
  } else {
    warning("Omega_beta not found in priors: restriction applied but Omega_beta not updated")
  }
  
  return(x)
}

#' Set up the steady-state BVAR model
#'
#' Prepares the components needed for prior specification and estimation. 
#' Also computes OLS estimates.
#'
#' @param x A steady-state \code{bvar} object created by \code{\link{bvar}}.
#' @param p Integer. The lag order of the VAR. Default \code{1}.
#' @param deterministic Character. The deterministic component to include.
#'   One of \code{"constant"} (default), \code{"constant_and_dummy"}, or
#'   \code{"constant_and_trend"}.
#' @param dummy Numeric vector of a dummy variable. Only
#'   used when \code{deterministic = "constant_and_dummy"}. Default \code{NULL}.
#'
#' @return The steady-state \code{bvar} object with a \code{setup} list containing the
#' required components for prior specification and estimation, and also the OLS estimates.
#' @export
#'
#' @examples
#' yt <- matrix(rnorm(50), 25, 2)
#' 
#' bvar_obj <- bvar(data = yt)
#' 
#' bvar_obj <- setup(bvar_obj)
setup <- function(x, p = 1, deterministic=c("constant", "constant_and_dummy", "constant_and_trend"), dummy=NULL) {
  
  if (!inherits(x, "bvar")) {
    stop("x must be a 'bvar' object")
  }
  
  if (is.null(x$data)) {
    stop("x must have data. Create bvar object with bvar(data = ...)")
  }
  
  if (!is.numeric(p) || p <= 0 || p != floor(p)) {
    stop("p must be a positive integer")
  }
  
  yt <- x$data
  
  if (p >= nrow(yt)) {
    stop("p must be less than the number of observations")
  }
  
  deterministic <- match.arg(deterministic)
  
  if (deterministic == "constant_and_dummy" && is.null(dummy)) {
    stop("dummy must be provided when deterministic = 'constant_and_dummy'")
  }
  
  if (!is.null(dummy) && length(dummy) != nrow(yt)) {
    stop("dummy must have length equal to number of observations")
  }
  
  if (!is.null(dummy) && !is.vector(dummy)) {
    stop("dummy must be a vector")
  }
  
  
  N = nrow(yt) - p
  k = ncol(yt)
  
  if (deterministic == "constant") {
    dt <- cbind(rep(1, nrow(yt)))
    q <- 1
  } else if (deterministic == "constant_and_dummy") {
    dt <- cbind(rep(1, nrow(yt)), dummy)
    q <- 2
  } else {
    trend <- 1:nrow(yt)
    dt <- cbind(rep(1, nrow(yt)), trend)
    q <- 2
  }
  
  Y <- yt[-c(1:p), ]
  W <- embed(yt, dimension = p+1)[, -(1:k)]
  X <- dt[-c(1:p), , drop=FALSE]
  Q <- embed(dt, dimension = p+1)[, -(1:q), drop=FALSE]
  
  Z <- cbind(W, X)
  beta_hat = solve(crossprod(Z), crossprod(Z, Y))
  U = Y - Z %*% beta_hat
  Sigma_u_OLS <- crossprod(U) / (N - k*p - q)
  
  if (q == 1) {
    C_hat <- beta_hat[(k*p+1):(k*p+q), ]
  } else {
    C_hat <- t(beta_hat[(k*p+1):(k*p+q), ])
  }
  
  A <- vector("list", p)
  for (i in 1:p) {
    rows_idx <- ((i - 1) * k + 1):(i * k)
    A[[i]] <- matrix(t(beta_hat[rows_idx, ]), nrow = k, ncol = k)
  }
  
  A_L <- diag(k)
  for (i in 1:p) {
    A_L <- A_L - A[[i]]
  }
  
  Psi_OLS <- solve(A_L, C_hat)
  beta_OLS = beta_hat[1:(k*p), ]
  
  n_free_params_A <- k*(k-1)/2
  
  Sigma_AR <- diag(0, k)
  
  for (i in 1:k){
    
    y <- yt[,i]
    
    Y_AR <- y[-c(1:p)]
    W_AR <- embed(y, dimension = p+1)[, -1]
    X_AR <- dt[-c(1:p), ,drop=FALSE]
    
    Z_AR <- cbind(W_AR,X_AR)
    beta_hat_AR = solve(crossprod(Z_AR,Z_AR),crossprod(Z_AR,Y_AR))
    U_AR = Y_AR-Z_AR%*%beta_hat_AR
    sigma2 <- crossprod(U_AR,U_AR)/(nrow(Z_AR)-ncol(Z_AR))
    Sigma_AR[i,i] <- sigma2
  }
  
  x$setup <- list(N=N,
                  k=k,
                  p=p,
                  Y=Y,
                  X=X,
                  W=W,
                  Q=Q,
                  q=q,
                  dummy=dummy,
                  beta_OLS=beta_OLS,
                  Sigma_u_OLS=Sigma_u_OLS,
                  Psi_OLS=Psi_OLS,
                  dt=dt,
                  D=X,
                  n_free_params_A=n_free_params_A,
                  Sigma_AR = Sigma_AR)
  return(x)
}

test_that("setup with constant deterministic", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  result <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  expect_s3_class(result, "bvar")
  expect_true(!is.null(result$setup))
  expect_equal(result$setup$k, 3)
  expect_equal(result$setup$p, 2)
  expect_equal(result$setup$q, 1)
})

test_that("setup with constant_and_trend", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  result <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant_and_trend")
  
  expect_equal(result$setup$q, 2)
  expect_equal(ncol(result$setup$dt), 2)  # constant + trend
})

test_that("setup with constant_and_dummy", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  dummy_var <- c(rep(0, 50), rep(1, 50))
  model <- bvar(data = data)
  result <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant_and_dummy", dummy = dummy_var)
  
  expect_equal(result$setup$q, 2)
  expect_equal(result$setup$dummy, dummy_var)
})

test_that("setup computes correct matrix dimensions", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  result <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  # N = nrow - p = 100 - 2 = 98
  expect_equal(result$setup$N, 98)
  expect_equal(nrow(result$setup$Y), 98)
  expect_equal(nrow(result$setup$X), 98)
  expect_equal(nrow(result$setup$W), 98)
})

test_that("setup computes OLS estimates", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  result <- SteadyStateBVAR::setup(model, p = 2, deterministic = "constant")
  
  expect_true(!is.null(result$setup$beta_OLS))
  expect_true(!is.null(result$setup$Sigma_u_OLS))
  expect_true(!is.null(result$setup$Psi_OLS))
  expect_equal(nrow(result$setup$Sigma_u_OLS), 3)
})

test_that("setup with different lag orders", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  
  result_p1 <- SteadyStateBVAR::setup(model, p = 1)
  result_p3 <- SteadyStateBVAR::setup(model, p = 3)
  
  expect_equal(result_p1$setup$p, 1)
  expect_equal(result_p3$setup$p, 3)
  expect_equal(result_p1$setup$N, 99)  # 100 - 1
  expect_equal(result_p3$setup$N, 97)  # 100 - 3
})

test_that("setup returns updated bvar object", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  result <- SteadyStateBVAR::setup(model, p = 2)
  
  expect_equal(result$data, model$data)  # data unchanged
  expect_true(!is.null(result$setup))
})

test_that("setup validates p is positive integer", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  
  expect_error(SteadyStateBVAR::setup(model, p = -1), "p must be a positive integer")
  expect_error(SteadyStateBVAR::setup(model, p = 0), "p must be a positive integer")
  expect_error(SteadyStateBVAR::setup(model, p = 2.5), "p must be a positive integer")
})

test_that("setup validates p is less than nrow", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  
  expect_error(SteadyStateBVAR::setup(model, p = 100), "p must be less than the number of observations")
})

test_that("setup requires data in bvar object", {
  model <- structure(
    list(
      data = NULL,
      setup = NULL,
      priors = NULL,
      fit = NULL,
      predict = list()
    ),
    class = "bvar"
  )
  
  expect_error(
    SteadyStateBVAR::setup(model, p = 2),
    "x must have data"
  )
})

test_that("setup requires dummy when constant_and_dummy", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  model <- bvar(data = data)
  
  expect_error(SteadyStateBVAR::setup(model, p = 2, deterministic = "constant_and_dummy"),
               "dummy must be provided")
})

test_that("setup validates dummy length", {
  data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  dummy_wrong <- c(rep(0, 50))  # Should be 100
  model <- bvar(data = data)
  
  expect_error(SteadyStateBVAR::setup(model, p = 2, deterministic = "constant_and_dummy", dummy = dummy_wrong),
               "dummy must have length equal to number of observations")
})

# Precompiled vignettes (too long runtime)

library(knitr)
# knit("vignettes/Homoscedastic-steady-state-BVAR.Rmd.orig", "vignettes/Homoscedastic-steady-state-BVAR.Rmd")
# fs::dir_copy("figure", "vignettes/figure", overwrite = TRUE)
# fs::dir_delete("figure")
knit("vignettes/RW-stochastic-volatility-steady-state-BVAR.Rmd.orig", "vignettes/RW-stochastic-volatility-steady-state-BVAR.Rmd")
fs::dir_copy("figure", "vignettes/figure", overwrite = TRUE)
fs::dir_delete("figure")
knit("vignettes/AR1-stochastic-volatility-steady-state-BVAR.Rmd.orig", "vignettes/AR1-stochastic-volatility-steady-state-BVAR.Rmd")
fs::dir_copy("figure", "vignettes/figure", overwrite = TRUE)
fs::dir_delete("figure")

#' @keywords internal
"_PACKAGE"
## usethis namespace: start
#' @importFrom grDevices rgb
#' @importFrom graphics abline axis lines par points polygon title legend
#' @importFrom stats embed frequency plot.ts qnorm quantile rnorm setNames start time ts ts.plot is.ts median
#' @importFrom utils head tail
#' @import Rcpp
#' @import methods
#' @importFrom rstan sampling
#' @importFrom rstantools rstan_config
#' @importFrom RcppParallel RcppParallelLibs
#' @useDynLib SteadyStateBVAR, .registration = TRUE
## usethis namespace: end
NULL

#' Conditional forecasts from a fitted steady-state BVAR model
#'
#' Computes and plots conditional forecasts from a fitted steady-state \code{bvar} object.
#' Conditions are imposed on specific variables at specific horizons using Algorithm 3.3.1
#' from Dieppe, Legrand, and van Roye (2016). Both conditional and unconditional
#' forecasts are plotted for comparison. Please note that for the moment, conditional forecasting
#' is only enabled for the homoscedastic steady-state BVAR, i.e. when \code{SV=FALSE} in \code{priors()}.
#'
#' @param bvar_obj A steady-state \code{bvar} object that has been passed through
#'   \code{\link{fit}}.
#' @param conditions A data frame with three columns: \code{var} (integer index
#'   of the conditioned variable), \code{horizon} (integer forecast horizon at
#'   which the condition is imposed), and \code{value} (numeric, the imposed
#'   condition). Note that \code{var} and \code{horizon} must be integers, as
#'   they are used for indexing.
#' @param pi Numeric. The prediction interval width. Default \code{0.95}, i.e. 95% prediction interval.
#' @param fcst_type Character. Whether to use \code{"mean"} or \code{"median"}
#'   as the point forecast. Default \code{"mean"}.
#' @param growth_rate_idx Integer vector. Indices of variables of which to convert forecasts to
#'   annual growth rates \eqn{\ln x_{t} - \ln x_{t-f}}, where \eqn{f} is
#'   the frequency of the data (4 for quarterly, 12 for monthly).
#'   Only suitable for variables specified as \eqn{\ln x_{t} - \ln x_{t-1}}, i.e.
#'   \code{diff(log(x))} or \code{100*diff(log(x))}.
#'   Computed by summing up to \eqn{f} log first differences. Default is \code{NULL}.
#' @param plot_idx Integer vector. Indices of variables to plot. If \code{NULL}
#'   (default), all variables are plotted. Forecasts are always computed and
#'   returned for all variables, regardless of \code{plot_idx}.
#'
#' @return Invisibly returns a list with three matrices: \code{forecast}, \code{lower}, and
#'   \code{upper}, each of dimension \code{H x k}.
#'   
#' @details
#' See Section 5.3 of Dieppe, Legrand, and van Roye (2016) for more details.
#' Please note the limitations of this method, see the detailed discussion in Section 5.4 of Dieppe, Legrand, and van Roye (2016).
#' 
#' @export
#'
#' @references
#' Dieppe, A., Legrand, R., and van Roye, B. (2016).
#' The BEAR toolbox. Working Paper Series, No. 1934. European Central Bank.
#'
#' @examples
#' \donttest{
#' #homoscedastic with Jeffreys prior
#' yt <- matrix(rnorm(50), 25, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    Jeffreys = TRUE,
#'                    SV = FALSE,
#'                    SV_type = NULL,
#'                    SV_priors = NULL)
#'                    
#' bvar_obj <- fit(bvar_obj,
#'                 H = 8,
#'                 d_pred = matrix(rep(1,8)),
#'                 iter = 200,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1)
#'                 
#' conditions <- data.frame(var = rep(2,8),
#'                          horizon = rep(1:8),
#'                          value   = rep(1,8))
#'                          
#' (cond_fcst <- conditional_forecast(bvar_obj,
#'                                   conditions,
#'                                   pi=0.68,
#'                                   fcst_type = "mean"))
#' }
conditional_forecast <- function(bvar_obj, conditions, pi = 0.95,
                                 fcst_type = c("mean", "median"),
                                 growth_rate_idx = NULL, plot_idx = NULL) {
  
  if (isTRUE(bvar_obj$priors$SV)) {
    stop("conditional_forecast is only supported for the homoscedastic steady-state BVAR (SV = FALSE).")
  }
  
  fcst_fun  <- match.arg(fcst_type)
  posterior <- rstan::extract(bvar_obj$fit$stan)
  n_draws   <- dim(posterior$beta)[1]
  
  p      <- bvar_obj$setup$p
  k      <- bvar_obj$setup$k
  Y      <- bvar_obj$setup$Y
  X      <- bvar_obj$setup$X
  N      <- bvar_obj$setup$N
  d_pred <- bvar_obj$predict$d_pred
  H      <- bvar_obj$predict$H
  alpha  <- 1 - pi
  s      <- k * H
  
  cond_forecast_array <- array(NA, dim = c(H, k, n_draws))
  
  for (n in 1:n_draws) {
    
    beta_n  <- posterior$beta[n, , ]
    Sigma_n <- posterior$Sigma_u[n, , ]
    Psi_n   <- posterior$Psi[n, , ]
    D_n     <- t(chol(Sigma_n))
    
    Pi_n <- vector("list", p)
    for (i in 1:p) {
      rows_idx <- ((i - 1) * k + 1):(i * k)
      Pi_n[[i]] <- t(beta_n[rows_idx, ])
    }
    
    Y_pred_mat <- matrix(NA, nrow = H, ncol = k)
    for (h in 1:H) {
      ytilde_t <- d_pred[h, ] %*% t(Psi_n)
      if (h > 1) {
        for (i in 1:min(h - 1, p)) {
          term     <- (Y_pred_mat[h - i, ] - d_pred[h - i, ] %*% t(Psi_n)) %*% t(Pi_n[[i]])
          ytilde_t <- ytilde_t + term
        }
      }
      if (h <= p) {
        for (i in h:p) {
          term     <- (Y[N + h - i, ] - X[N + h - i, ] %*% t(Psi_n)) %*% t(Pi_n[[i]])
          ytilde_t <- ytilde_t + term
        }
      }
      Y_pred_mat[h, ] <- ytilde_t
    }
    
    Pi_n        <- t(beta_n)
    Phi_n       <- MTS::VARpsi(Pi_n, lag = H)$psi
    n_Phi       <- ncol(Phi_n) / k
    Phi_n_tilde <- lapply(1:n_Phi, function(i) {
      cols <- ((i - 1) * k + 1):(i * k)
      Phi_n[, cols] %*% D_n
    })
    
    v <- nrow(conditions)
    R <- matrix(0, v, s)
    r <- rep(0, v)
    
    for (row_idx in 1:v) {
      i     <- conditions$var[row_idx]
      h     <- conditions$horizon[row_idx]
      value <- conditions$value[row_idx]
      row   <- c()
      for (j in 1:h) {
        row <- c(row, Phi_n_tilde[[h - j + 1]][i, ])
      }
      R[row_idx, 1:length(row)] <- row
      r[row_idx] <- value - Y_pred_mat[h, i]
    }
    
    svd_R  <- svd(R, nu = v, nv = s)
    U      <- svd_R$u
    D      <- diag(svd_R$d)
    V_1    <- svd_R$v[, 1:v]
    V_2    <- svd_R$v[, (v + 1):s]
    lambda <- rnorm(s - v)
    eta    <- V_1 %*% solve(D) %*% t(U) %*% r + V_2 %*% lambda
    
    eta_mat    <- matrix(eta, nrow = k, ncol = H)
    Y_cond_mat <- matrix(NA, nrow = H, ncol = k)
    for (h in 1:H) {
      sum_shocks <- matrix(0, nrow = k, ncol = 1)
      for (j in 1:h) {
        sum_shocks <- sum_shocks + Phi_n_tilde[[h - j + 1]] %*% eta_mat[, j]
      }
      Y_cond_mat[h, ] <- Y_pred_mat[h, ] + sum_shocks
    }
    cond_forecast_array[, , n] <- Y_cond_mat
  }
  
  cond_point <- apply(cond_forecast_array, c(1, 2), fcst_fun)
  cond_lower <- apply(cond_forecast_array, c(1, 2), quantile, probs = alpha / 2)
  cond_upper <- apply(cond_forecast_array, c(1, 2), quantile, probs = 1 - alpha / 2)
  
  y_pred_uncond   <- posterior$y_pred
  y_pred_m_uncond <- apply(y_pred_uncond, c(2, 3), fcst_fun)
  
  data_Y    <- bvar_obj$data
  freq      <- frequency(data_Y)
  m         <- ncol(data_Y)
  if (is.null(plot_idx)) plot_idx <- 1:m
  
  time_hist <- as.numeric(time(data_Y))
  time_fore <- seq(tail(time_hist, 1) + 1 / freq, by = 1 / freq, length.out = H)
  
  forecast_ret <- matrix(NA, H, m)
  lower_ret    <- matrix(NA, H, m)
  upper_ret    <- matrix(NA, H, m)
  colnames(forecast_ret) <- colnames(data_Y)
  colnames(lower_ret)    <- colnames(data_Y)
  colnames(upper_ret)    <- colnames(data_Y)
  
  for (i in 1:m) {
    smply      <- data_Y[, i]
    fcst_m     <- cond_point[, i]
    fcst_lower <- cond_lower[, i]
    fcst_upper <- cond_upper[, i]
    uncond_m   <- y_pred_m_uncond[, i]
    
    if (!is.null(growth_rate_idx) && i %in% growth_rate_idx) {
      annual_hist <- rep(NA, length(smply))
      for (t in freq:length(smply)) {
        annual_hist[t] <- sum(smply[(t - (freq - 1)):t])
      }
      annual_hist        <- ts(annual_hist, start = start(data_Y), frequency = freq)
      last_obs           <- tail(smply, (freq - 1))
      all_fcst           <- c(last_obs, fcst_m)
      all_fcst_uncond    <- c(last_obs, uncond_m)
      annual_fcst        <- rep(NA, H)
      annual_fcst_uncond <- rep(NA, H)
      for (t_h in 1:H) {
        annual_fcst[t_h]        <- sum(all_fcst[t_h:(t_h + (freq - 1))])
        annual_fcst_uncond[t_h] <- sum(all_fcst_uncond[t_h:(t_h + (freq - 1))])
      }
      all_lower    <- c(last_obs, fcst_lower)
      all_upper    <- c(last_obs, fcst_upper)
      annual_lower <- rep(NA, H)
      annual_upper <- rep(NA, H)
      for (t_h in 1:H) {
        annual_lower[t_h] <- sum(all_lower[t_h:(t_h + (freq - 1))])
        annual_upper[t_h] <- sum(all_upper[t_h:(t_h + (freq - 1))])
      }
      forecast_ret[, i] <- annual_fcst
      lower_ret[, i]    <- annual_lower
      upper_ret[, i]    <- annual_upper
      
      if (i %in% plot_idx) {
        time_full   <- c(tail(time_hist, 1), time_fore)
        m_full      <- c(tail(annual_hist, 1), annual_fcst)
        lower_full  <- c(tail(annual_hist, 1), annual_lower)
        upper_full  <- c(tail(annual_hist, 1), annual_upper)
        uncond_full <- c(tail(annual_hist, 1), annual_fcst_uncond)
        plot.ts(annual_hist, main = paste(colnames(data_Y)[i], "(annual)"),
                xlab = "Time", ylab = NULL, col = "black", lwd = 2,
                xlim = c(head(time_hist, 1), tail(time_fore, 1)),
                ylim = range(upper_full, lower_full, annual_hist, uncond_full, na.rm = TRUE))
        polygon(c(time_full, rev(time_full)), c(upper_full, rev(lower_full)),
                col = rgb(0, 0, 1, 0.2), border = NA)
        lines(time_full, m_full,      col = "blue", lwd = 2)
        lines(time_full, uncond_full, col = "red",  lwd = 1, lty = 1)
        legend("bottomleft",
               legend = c("Conditional forecast", "Unconditional forecast"),
               col    = c("blue", "red"),
               lwd    = 2,
               bty    = "n")
      }
    } else {
      forecast_ret[, i] <- fcst_m
      lower_ret[, i]    <- fcst_lower
      upper_ret[, i]    <- fcst_upper
      
      if (i %in% plot_idx) {
        time_full   <- c(tail(time_hist, 1), time_fore)
        m_full      <- c(tail(smply, 1), fcst_m)
        lower_full  <- c(tail(smply, 1), fcst_lower)
        upper_full  <- c(tail(smply, 1), fcst_upper)
        uncond_full <- c(tail(smply, 1), uncond_m)
        plot.ts(smply, main = colnames(data_Y)[i], xlab = "Time", ylab = NULL,
                col = "black", lwd = 2,
                xlim = c(head(time_hist, 1), tail(time_fore, 1)),
                ylim = range(upper_full, lower_full, smply, uncond_full))
        polygon(c(time_full, rev(time_full)), c(upper_full, rev(lower_full)),
                col = rgb(0, 0, 1, 0.2), border = NA)
        lines(time_full, m_full,      col = "blue", lwd = 2)
        lines(time_full, uncond_full, col = "red",  lwd = 1, lty = 1)
        legend("bottomleft",
               legend = c("Conditional forecast", "Unconditional forecast"),
               col    = c("blue", "red"),
               lwd    = 2,
               bty    = "n")
      }
    }
  }
  
  invisible(list(
    forecast   = forecast_ret,
    lower      = lower_ret,
    upper      = upper_ret
  ))
}

#' Create a steady-state BVAR model object
#'
#' Initialises a steady-state \code{bvar} object. This is the starting point for all models in \code{SteadyStateBVAR}.
#' After creation, pass the object sequentially to \code{\link{setup}}, \code{\link{priors}},
#' and \code{\link{fit}} to build and estimate the model.
#' 
#' @details
#' The model takes the form
#'
#' \deqn{y_t = \Psi d_t + \Pi_1(y_{t-1}-\Psi d_{t-1})+\dots+\Pi_p(y_{t-p}-\Psi d_{t-p})+u_t}
#'
#' where \eqn{y_t} is an \eqn{k}-dimensional vector of endogenous variables at time \eqn{t}, and \eqn{d_t} is
#' a \eqn{q}-dimensional vector of deterministic (exogenous) variables at time \eqn{t}.
#' Here \eqn{\Pi_\ell} for \eqn{\ell=1,\dots,p} is a \eqn{(k \times k)} matrix of autoregressive parameters,
#' and \eqn{\Psi} is a \eqn{(k \times q)} matrix of steady-state parameters. Note that
#' \eqn{\mathrm{E}(y_t)=\mu_t=\Psi d_t} is the unconditional mean, or the \strong{steady state} of the process.
#' One can stack the (transposed) \eqn{\Pi_i} matrices in the \eqn{(kp \times k)} matrix \eqn{\beta}
#' \deqn{\beta=\begin{bmatrix}\Pi'_1 \\ \vdots  \\\Pi'_p\end{bmatrix}}
#' Then the model can be rewritten as a nonlinear regression (Karlsson, 2013)
#' \deqn{y_t' =d_t'\Psi' + \left[w_t'-q_t'(I_p \otimes \Psi') \right]\beta +u_t'}
#' where where \eqn{w_t'=(y_{t-1}',\dots,y_{t-p}')} is a \eqn{kp}-dimensional vector of lagged endogenous variables
#' and \eqn{q_t'=(d_{t-1}',\dots,d_{t-p}')} is a \eqn{qp}-dimensional vector of lagged deterministic (exogenous) variables,
#' \eqn{I_p} is the \eqn{(p \times p)} identity matrix and \eqn{\otimes} denotes the Kronecker product.
#' This is how the likelihood is written in the Stan code. The goal is to estimate \eqn{\beta, \Psi}, and \eqn{\Sigma_u}.
#' 
#' For the innovations to the model, in the case of the homoscedastic steady-state BVAR, they are \eqn{u_t \overset{\text{iid}}{\sim} \mathrm{N_k}(0,\Sigma_u)}.
#' However, for models with stochastic volatility, there is instead a time-varying covariance matrix \eqn{u_t \sim \mathrm{N_k}(0,\Sigma_{u,t})}.
#' The innovations then take the form
#' 
#' \deqn{\begin{aligned} u_t &= A^{-1} \Lambda^{0.5}_t \epsilon_t \\
#' \epsilon_t &\overset{\text{iid}}{\sim} \mathrm{N}(0, \mathrm{I}_k)\end{aligned}}
#' 
#' where \eqn{A} is a lower triangular matrix with ones on the diagonal that describes 
#' the contemporaneous interaction of the endogenous variables, and
#' 
#' \deqn{\Lambda_t = \mathrm{diag}(\lambda_{1,t},\dots,\lambda_{k,t})}
#' 
#' contains the time-varying volatilities.
#' For the \code{AR1} stochastic volatility specification, the log volatilities follow AR(1) processes
#' 
#' \deqn{\ln \lambda_{i,t} = \gamma_{0,i} + \gamma_{1,i} \ln \lambda_{i,t-1} + \nu_{i,t}, \ i=1,\dots,k}
#' 
#' where the log volatility AR(1) processes are restricted to the stationary region, i.e. \eqn{|\gamma_{1,i}|<1 \ \forall i}.
#' For the \code{RW} stochastic volatility specification, the log volatilities follow Random Walk processes
#' 
#' \deqn{\gamma_{0,i}=0, \ \gamma_{1,i}=1 \ \forall i}
#' 
#' The innovations to the log volatilities follow in the \code{AR1} case
#' 
#' \deqn{\nu_{t} = (\nu_{1,t},\dots,\nu_{k,t})'\overset{\text{iid}}{\sim} \mathrm{N}(0, \Phi)}
#' 
#' where \eqn{\Phi} \emph{is not diagonal} and as such the innovations to the log
#' volatilities are allowed to be correlated across variables. For the \code{RW} case, \eqn{\Phi} \emph{is diagonal}
#' with variances \eqn{\phi_i} for \eqn{i=1,\dots,k}.
#' 
#' Note that under both stochastic volatility specifications, the time-varying covariance matrix is
#' 
#' \deqn{\Sigma_{u,t} = A^{-1} \Lambda_t (A^{-1})'}
#' 
#' For details on the homoscedastic steady-state BVAR model, see Villani (2009).
#' For details on the Random Walk stochastic volatility steady-state BVAR model, see Clark (2011).
#' See Carriero, Clark, and Marcellino (2024) for the above-mentioned AR(1) stochastic volatility
#' specification applied to a conventional BVAR.
#' 
#'
#' @param data A numeric matrix or time series of data where each column is a
#'   variable and each row is a time period.
#'
#' @return An object of class \code{bvar}.
#' 
#' @references
#' Carriero, A., Clark, T. E., and Marcellino, M. (2024).
#' Capturing macro-economic tail risks with Bayesian vector autoregressions. 
#' \emph{Journal of Money, Credit and Banking}, 56(5), pp. 1099–1127.
#' 
#' Clark, T. E. (2011). Real-time density forecasts from Bayesian vector autoregressions
#' with stochastic volatility. \emph{Journal of Business & Economic Statistics}, 29(3), pp. 327–341.
#' 
#' Karlsson, S. (2013). Forecasting with Bayesian vector autoregression.
#' In: Elliott, G. and Timmermann, A. (eds), \emph{Handbook of Economic Forecasting}.
#' Elsevier B.V., Vol. 2, Part B, pp. 791–897.
#' 
#' Villani, M. (2009). Steady-state priors for vector autoregressions.
#' \emph{Journal of Applied Econometrics}, 24(4), pp. 630–650. 
#' 
#'
#'
#' @export
#'
#' @examples
#' yt <- matrix(rnorm(50), 25, 2)
#' 
#' bvar_obj <- bvar(data = yt)
bvar <- function(data) {
  
  if (!is.matrix(data) && !is.ts(data)) {
    stop("data must be a matrix or time series object")
  }
  
  obj <- list(
    data    = data,
    setup   = NULL,
    priors  = NULL,
    fit     = NULL,
    predict = list()
  )
  
  class(obj) <- "bvar"
  return(obj)
}

test_that("bvar requires data argument", {
  expect_error(bvar(), "missing")
})

test_that("bvar validates data input", {
  expect_error(bvar(data = "not_data"), 
               "data must be a matrix or time series object")
})

test_that("bvar creates object with correct class", {
  test_data <- matrix(rnorm(100), nrow = 20, ncol = 5)
  model <- bvar(data = test_data)
  expect_s3_class(model, "bvar")
})

test_that("bvar stores data correctly", {
  test_data <- matrix(rnorm(300), nrow = 100, ncol = 3)
  colnames(test_data) <- c("GDP", "Inflation", "Rate")
  
  model <- bvar(data = test_data)
  expect_equal(model$data, test_data)
  expect_equal(nrow(model$data), 100)
  expect_equal(ncol(model$data), 3)
})

test_that("bvar initializes component slots", {
  test_data <- matrix(rnorm(100), nrow = 20, ncol = 5)
  model <- bvar(data = test_data)
  
  expect_null(model$setup)
  expect_null(model$priors)
  expect_null(model$fit)
  expect_type(model$predict, "list")
})

#' Koop and Korobilis (2010) dataset
#'
#' Quarterly US macroeconomic data (1953Q1-2006Q3) from Koop and Korobilis (2010)
#'
#' @format ## `KoopKorobilis2010`
#' A multivariate time series object with 215 rows and 3 columns:
#' \describe{
#'   \item{delta pi}{inflation rate (annual percentage change in a chain-weighted GDP price index)}
#'   \item{u}{unemployment rate (seasonally adjusted civilian unemployment rate, all civilian workers aged 16 years or older)}
#'   \item{r}{interest rate (yield on the three-month Treasury bill rate)}
#' }
#' @source Koop, G. and Korobilis, D. (2010). Bayesian multivariate time series
#'  methods for empirical macroeconomics. \emph{Foundations and Trends in Econometrics}, 3(4), pp. 267-358.
"KoopKorobilis2010"

#' Villani (2009) dataset
#'
#' Swedish macroeconomic data (1980Q1–2005Q4) from Section 4.1 in Villani (2009)
#'
#' @format ## `Villani2009`
#' A multivariate time series object with 104 rows and 7 columns:
#' \describe{
#'   \item{delta y_f}{foreign GDP growth}
#'   \item{pi_f}{foreign CPI inflation}
#'   \item{i_f}{foreign 3-month interest rate}
#'   \item{delta y}{domestic GDP growth}
#'   \item{pi}{domestic CPI inflation}
#'   \item{i}{domestic 3-month interest rate}
#'   \item{q}{level of the real exchange rate defined as \eqn{q = s + p_f - p}, where \eqn{p_f} and \eqn{p}
#'   are the foreign and domestic CPI levels in logs and \eqn{s} is the log of the trade-weighted nominal exchange rate}
#' }
#' @source Villani, M. (2009). Steady-state priors for vector autoregressions.
#'   \emph{Journal of Applied Econometrics}, 24(4), pp. 630-650.
"Villani2009"

#' Estimate the steady-state BVAR model using Stan
#'
#' Estimates the steady-state BVAR model using the No-U-Turn sampler (a variant of Hamiltonian Monte Carlo) via Stan.
#' Also generates draws from the joint predictive distribution.
#' Uses the data, setup, and priors stored in the steady-state \code{bvar} object.
#'
#' @param x A steady-state \code{bvar} object that has been passed through
#'   \code{\link{setup}} and \code{\link{priors}}.
#' @param H Positive Integer. Forecast horizon.
#'   Default is \code{1}.
#' @param d_pred Matrix of size \eqn{H \times q}. Future values of the deterministic variables \eqn{d_t}. Default is \code{NULL}, must be provided by the user.
#' @param ... Additional arguments passed directly to the \code{rstan} function \code{\link[rstan]{sampling}}
#' (e.g. \code{iter}, \code{warmup}, \code{chains}, \code{cores}, \code{control},
#' \code{seed}, \code{init}, \code{thin}, \code{algorithm}, \code{pars},
#' \code{include}, \code{refresh}, \code{verbose}, \code{save_warmup},
#' \code{sample_file}, \code{diagnostic_file}). If \code{pars}/\code{include} is
#' used to exclude model parameters required by \code{fit()} for posterior
#' summaries, an error will be raised.
#'
#' @return A fitted steady-state \code{bvar} object with:
#' \itemize{
#' \item \code{x$fit$stan}: An object of class \code{stanfit}
#' \item \code{x$fit$posterior_means}: List of posterior mean estimates
#' \item \code{x$fit$posterior_medians}: List of posterior median estimates
#' }
#'
#' @details
#' The function selects the appropriate precompiled Stan model based on settings from \link{priors}:
#' \itemize{
#'   \item \code{steady_state_bvar_homoscedastic_jeffreys_prior}
#'   \item \code{steady_state_bvar_homoscedastic_inverse_wishart_prior}
#'   \item \code{steady_state_bvar_RW_stochastic_volatility}
#'   \item \code{steady_state_bvar_AR1_stochastic_volatility}
#' }
#' The function estimates the following parameters (see \link{bvar} for details):
#' \itemize{
#'       \item \code{beta}: \eqn{kp \times k} VAR coefficient matrix
#'       \item \code{Psi}: \eqn{k \times q} steady-state parameter matrix
#'       \item \code{Sigma_u}: innovation covariance matrix (\eqn{k \times k} for homoscedastic,
#'         \eqn{T \times k \times k} for stochastic volatility)
#'       \item If Random Walk stochastic volatility:
#'        \itemize{
#'        \item \code{A}: \eqn{k \times k} lower triangular matrix with ones on the diagonal that describes
#'         the contemporaneous interaction of the endogenous variables
#'        \item \code{phi}: \eqn{k}-dimensional vector of log volatility innovation variances
#'        }
#'       \item If AR1 stochastic volatility:
#'        \itemize{
#'        \item \code{A}: \eqn{k \times k} lower triangular matrix with ones on the diagonal that describes
#'         the contemporaneous interaction of the endogenous variables
#'         \item \code{gamma_0}: \eqn{k}-dimensional vector of log volatility intercepts
#'         \item \code{gamma_1}: \eqn{k}-dimensional vector of log volatility slopes
#'         \item \code{Phi}: \eqn{k \times k} log volatility innovation covariance matrix
#'        }
#'     }
#'
#' @export
#' @examples
#' \donttest{
#' #homoscedastic with Jeffreys prior, d_t = constant
#' 
#' yt <- matrix(rnorm(20), 10, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p = 1, deterministic = "constant")
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    Jeffreys = TRUE)
#'                    
#' H <- 8
#' d_pred <- matrix(rep(1,8))
#' colnames(d_pred) <- c("constant")
#' rownames(d_pred) <- paste("Horizon", 1:H)
#' print(d_pred)
#' 
#' bvar_obj <- fit(bvar_obj,
#'                 H = H,
#'                 d_pred = d_pred,
#'                 iter = 200,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1)
#'                 
#'                 
#' #homoscedastic with inverse-Wishart prior, d_t = constant and dummy
#' 
#' yt <- matrix(rnorm(20), 10, 2)
#'
#' bvar_obj <- bvar(data = yt)
#' 
#' dummy_variable <- c(rep(1,5), rep(0,5))
#'
#' bvar_obj <- setup(bvar_obj, p = 1,
#'                   deterministic = "constant_and_dummy",
#'                   dummy = dummy_variable)
#'                   
#' k <- bvar_obj$setup$k
#' q <- bvar_obj$setup$q
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, k*q),
#'                    Omega_Psi = diag(0.1, k*q, k*q),
#'                    Jeffreys = FALSE) #inverse-Wishart
#'
#' H <- 8
#' d_pred <- cbind(rep(1, H), rep(0, H))
#' colnames(d_pred) <- c("constant", "dummy")
#' rownames(d_pred) <- paste("Horizon", 1:H)
#' print(d_pred)
#'
#' bvar_obj <- fit(bvar_obj,
#'                 H = H,
#'                 d_pred = d_pred,
#'                 iter = 200,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1)
#'  
#'
#' #RW stochastic volatility
#' 
#' yt <- matrix(rnorm(20), 10, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")
#'
#' k <- bvar_obj$setup$k
#' n_free_params_A <- bvar_obj$setup$n_free_params_A
#'
#' SV_priors_RW <- list(
#' theta_A              =  rep(0, n_free_params_A),
#' Omega_A              =  diag(1000, n_free_params_A),
#' mu_log_lambda_1      =  rep(0, k),
#' sigma2_log_lambda_1  =  rep(1000, k),
#' alpha_phi            =  rep(5, k),
#' beta_phi             = (rep(5, k) - 1) * rep(0.1, k)
#' )
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    SV = TRUE,
#'                    SV_type = "RW",
#'                    SV_priors = SV_priors_RW)
#'
#' bvar_obj <- fit(bvar_obj,
#'                 H = 8,
#'                 d_pred = matrix(rep(1,8)),
#'                 iter = 200,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1,
#'                 control = list(max_treedepth = 12, adapt_delta = 0.85)
#'                 )
#'
#'
#' #AR1 stochastic volatility
#' 
#' yt <- matrix(rnorm(20), 10, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")
#'
#' k <- bvar_obj$setup$k
#' n_free_params_A <- bvar_obj$setup$n_free_params_A
#'
#' SV_priors_AR1 <- list(
#' theta_A               =  rep(0, n_free_params_A),
#' Omega_A               =  diag(1000, n_free_params_A),
#' theta_gamma_0         =  rep(0.1, k),
#' Omega_gamma_0         =  diag(1000, k),
#' theta_gamma_1         =  rep(0.9, k),
#' Omega_gamma_1         =  diag(10, k),
#' theta_log_lambda_1    =  rep(0.1, k)/(1-rep(0.9, k)),
#' Omega_log_lambda_1    =  diag(1000, k),
#' V_Phi                 = (10 - k - 1) * diag(k),
#' m_Phi                 =  10
#' )
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    SV = TRUE,
#'                    SV_type = "AR1",
#'                    SV_priors = SV_priors_AR1)
#'
#' bvar_obj <- fit(bvar_obj,
#'                 H = 8,
#'                 d_pred = matrix(rep(1,8)),
#'                 iter = 200,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1,
#'                 control = list(max_treedepth = 12, adapt_delta = 0.85)
#'                 )
#' }
fit <- function(x, H = 1, d_pred = NULL, ...) {
  
  if (!inherits(x, "bvar")) stop("must be a 'bvar' object")
  if (is.null(x$setup)) stop("must be passed through setup")
  if (is.null(x$priors)) stop("must be passed through priors")
  if (is.null(d_pred)) stop("d_pred must be supplied")
  
  if (!is.numeric(H) || length(H) != 1 || !is.finite(H) || H < 1 || H != floor(H)) {
    stop("H must be a positive integer")
  }
  if (!is.matrix(d_pred)) stop("d_pred must be a matrix")
  if (nrow(d_pred) != H) stop("nrow(d_pred) must equal H")
  if (!is.null(x$setup$q) && ncol(d_pred) != x$setup$q) {
    stop("ncol(d_pred) must equal q")
  }
  
  x$predict$H <- H
  x$predict$d_pred <- d_pred
  
  setup <- x$setup
  priors <- x$priors
  SV <- isTRUE(priors$SV)
  SV_type <- priors$SV_type
  
  stan_data <- c(setup, list(H = H, d_pred = d_pred), priors)
  
  model_name <- if (isFALSE(priors$Jeffreys)) {
    "steady_state_bvar_homoscedastic_inverse_wishart_prior"
  } else {
    "steady_state_bvar_homoscedastic_jeffreys_prior"
  }
  
  if (isTRUE(SV)) {
    if (is.null(SV_type)) stop("SV_type missing in priors")
    model_name <- if (SV_type == "RW") {
      "steady_state_bvar_RW_stochastic_volatility"
    } else if (SV_type == "AR1") {
      "steady_state_bvar_AR1_stochastic_volatility"
    } else {
      stop("SV_type must be 'RW' or 'AR1'")
    }
    stan_data <- c(priors$SV_priors, stan_data)
    if (setup$k == 2 && !is.null(priors$SV_priors$theta_A)) {
      stan_data$theta_A <- as.array(priors$SV_priors$theta_A[1])
    }
  }
  
  x$fit$stan <- rstan::sampling(stanmodels[[model_name]], data = stan_data, ...)
  
  posterior <- rstan::extract(x$fit$stan)
  
  required_params <- if (!SV) {
    c("beta", "Psi", "Sigma_u")
  } else if (SV_type == "AR1") {
    c("beta", "Psi", "A", "gamma_0", "gamma_1", "Phi", "Sigma_u")
  } else {
    c("beta", "Psi", "A", "phi", "Sigma_u")
  }
  
  missing_params <- setdiff(required_params, names(posterior))
  if (length(missing_params) > 0) {
    stop(sprintf(
      "The following required parameters were not found in the Stan output: %s. This likely happened because 'pars'/'include' was passed via ... and excluded them.",
      paste(missing_params, collapse = ", ")
    ))
  }
  
  x$fit$posterior_means <- list()
  x$fit$posterior_medians <- list()
  
  if (!SV) {
    
    x$fit$posterior_means$beta <- apply(posterior$beta, c(2, 3), mean)
    x$fit$posterior_means$Psi <- apply(posterior$Psi, c(2, 3), mean)
    x$fit$posterior_means$Sigma_u <- apply(posterior$Sigma_u, c(2, 3), mean)
    
    x$fit$posterior_medians$beta <- apply(posterior$beta, c(2, 3), median)
    x$fit$posterior_medians$Psi <- apply(posterior$Psi, c(2, 3), median)
    x$fit$posterior_medians$Sigma_u <- apply(posterior$Sigma_u, c(2, 3), median)
    
  } else if (SV_type == "AR1") {
    
    x$fit$posterior_means$beta <- apply(posterior$beta, c(2, 3), mean)
    x$fit$posterior_means$Psi <- apply(posterior$Psi, c(2, 3), mean)
    x$fit$posterior_means$A <- apply(posterior$A, c(2, 3), mean)
    x$fit$posterior_means$gamma_0 <- apply(posterior$gamma_0, 2, mean)
    x$fit$posterior_means$gamma_1 <- apply(posterior$gamma_1, 2, mean)
    x$fit$posterior_means$Phi <- apply(posterior$Phi, c(2, 3), mean)
    x$fit$posterior_means$Sigma_u <- apply(posterior$Sigma_u, c(2, 3, 4), mean)
    
    x$fit$posterior_medians$beta <- apply(posterior$beta, c(2, 3), median)
    x$fit$posterior_medians$Psi <- apply(posterior$Psi, c(2, 3), median)
    x$fit$posterior_medians$A <- apply(posterior$A, c(2, 3), median)
    x$fit$posterior_medians$gamma_0 <- apply(posterior$gamma_0, 2, median)
    x$fit$posterior_medians$gamma_1 <- apply(posterior$gamma_1, 2, median)
    x$fit$posterior_medians$Phi <- apply(posterior$Phi, c(2, 3), median)
    x$fit$posterior_medians$Sigma_u <- apply(posterior$Sigma_u, c(2, 3, 4), median)
    
  } else if (SV_type == "RW") {
    
    x$fit$posterior_means$beta <- apply(posterior$beta, c(2, 3), mean)
    x$fit$posterior_means$Psi <- apply(posterior$Psi, c(2, 3), mean)
    x$fit$posterior_means$A <- apply(posterior$A, c(2, 3), mean)
    x$fit$posterior_means$phi <- apply(posterior$phi, 2, mean)
    x$fit$posterior_means$Sigma_u <- apply(posterior$Sigma_u, c(2, 3, 4), mean)
    
    x$fit$posterior_medians$beta <- apply(posterior$beta, c(2, 3), median)
    x$fit$posterior_medians$Psi <- apply(posterior$Psi, c(2, 3), median)
    x$fit$posterior_medians$A <- apply(posterior$A, c(2, 3), median)
    x$fit$posterior_medians$phi <- apply(posterior$phi, 2, median)
    x$fit$posterior_medians$Sigma_u <- apply(posterior$Sigma_u, c(2, 3, 4), median)
  }
  
  return(x)
}

#' Forecast from a fitted steady-state BVAR model
#'
#' Computes and plots forecasts from a fitted \code{bvar} object. Draws from the 
#' joint predictive distribution are used to construct point forecasts and
#' prediction intervals. Optionally converts monthly or quarterly growth rate forecasts to annual growth
#' rates for selected variables.
#'
#' @param x A steady-state \code{bvar} object that has been passed through \code{\link{fit}}.
#' @param pi Numeric. The prediction interval width. Default \code{0.95}, i.e. 95% prediction interval.
#' @param fcst_type Character. Whether to use \code{"mean"} or \code{"median"}
#'   as the point forecast. Default \code{"mean"}.
#' @param growth_rate_idx Integer vector. Indices of variables of which to convert forecasts to
#'   annual growth rates \eqn{\ln x_{t} - \ln x_{t-f}}, where \eqn{f} is
#'   the frequency of the data (4 for quarterly, 12 for monthly).
#'   Only suitable for variables specified as \eqn{\ln x_{t} - \ln x_{t-1}}, i.e.
#'   \code{diff(log(x))} or \code{100*diff(log(x))}.
#'   Computed by summing up to \eqn{f} log first differences. Default is \code{NULL}.
#' @param plot_idx Integer vector. Indices of variables to plot. If \code{NULL}
#'   (default), all variables are plotted. Forecasts are always computed and
#'   returned for all variables, regardless of \code{plot_idx}.
#' @param show_all Logical. If \code{FALSE} (default), only the last two years
#'   of history are shown alongside the forecast. If \code{TRUE}, the full
#'   history is shown.
#'
#' @return Invisibly returns a list with three matrices: \code{forecast}, \code{lower}, and
#'   \code{upper}, each of dimension \code{H x k} where \code{H} is the
#'   forecast horizon and \code{k} is the number of variables.
#' @export
#'
#' @examples
#' \donttest{
#' #homoscedastic with Jeffreys prior
#' yt <- matrix(rnorm(50), 25, 2)
#'
#' bvar_obj <- bvar(data = yt)
#'
#' bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")
#'
#' bvar_obj <- priors(bvar_obj,
#'                    lambda_1 = 0.2,
#'                    lambda_2 = 0.5,
#'                    lambda_3 = 1,
#'                    first_own_lag_prior_mean = rep(1,2),
#'                    theta_Psi = rep(0, 2),
#'                    Omega_Psi = diag(0.1, 2, 2),
#'                    Jeffreys = TRUE,
#'                    SV = FALSE,
#'                    SV_type = NULL,
#'                    SV_priors = NULL)
#'                    
#' bvar_obj <- fit(bvar_obj,
#'                 H = 8,
#'                 d_pred = matrix(rep(1,8)),
#'                 iter = 200,
#'                 warmup = 50,
#'                 chains = 1,
#'                 cores = 1)
#'
#' (fcst <- forecast(bvar_obj, pi = 0.90, show_all = TRUE))
#' }
forecast <- function(x, pi = 0.95, fcst_type = c("mean", "median"),
                     growth_rate_idx = NULL, plot_idx = NULL, show_all = FALSE) {
  
  fcst_type <- match.arg(fcst_type)
  
  Y    <- x$data
  freq <- frequency(Y)
  
  if (is.null(plot_idx))
    plot_idx <- 1:ncol(Y)
  
  posterior    <- rstan::extract(x$fit$stan)
  y_pred       <- posterior$y_pred
  alpha        <- 1 - pi
  y_pred_m     <- apply(y_pred, c(2, 3), fcst_type)
  y_pred_lower <- apply(y_pred, c(2, 3), quantile, probs = alpha / 2)
  y_pred_upper <- apply(y_pred, c(2, 3), quantile, probs = 1 - alpha / 2)
  
  T <- nrow(Y)
  H <- nrow(y_pred_m)
  m <- ncol(Y)
  
  forecast_ret <- matrix(NA, H, m)
  lower_ret    <- matrix(NA, H, m)
  upper_ret    <- matrix(NA, H, m)
  colnames(forecast_ret) <- colnames(Y)
  colnames(lower_ret)    <- colnames(Y)
  colnames(upper_ret)    <- colnames(Y)
  
  time_hist <- as.numeric(time(Y))
  time_fore <- seq(tail(time_hist, 1) + 1 / freq, by = 1 / freq, length.out = H)
  
  for (i in 1:m) {
    smply      <- Y[, i]
    fcst_m     <- y_pred_m[, i]
    fcst_lower <- y_pred_lower[, i]
    fcst_upper <- y_pred_upper[, i]
    
    if (!is.null(growth_rate_idx) && i %in% growth_rate_idx) {
      
      annual_hist <- rep(NA, length(smply))
      for (t in freq:length(smply)) {
        annual_hist[t] <- sum(smply[(t - (freq - 1)):t])
      }
      annual_hist <- ts(annual_hist, start = start(Y), frequency = freq)
      
      last_obs   <- tail(smply, (freq - 1))
      all_fcst   <- c(last_obs, fcst_m)
      all_lower  <- c(last_obs, fcst_lower)
      all_upper  <- c(last_obs, fcst_upper)
      
      annual_fcst  <- rep(NA, H)
      annual_lower <- rep(NA, H)
      annual_upper <- rep(NA, H)
      
      for (t_h in 1:H) {
        annual_fcst[t_h]  <- sum(all_fcst[t_h:(t_h + (freq - 1))])
        annual_lower[t_h] <- sum(all_lower[t_h:(t_h + (freq - 1))])
        annual_upper[t_h] <- sum(all_upper[t_h:(t_h + (freq - 1))])
      }
      
      forecast_ret[, i] <- annual_fcst
      lower_ret[, i]    <- annual_lower
      upper_ret[, i]    <- annual_upper
      
      if (i %in% plot_idx) {
        time_full  <- c(tail(time_hist, 1), time_fore)
        m_full     <- c(tail(annual_hist, 1), annual_fcst)
        lower_full <- c(tail(annual_hist, 1), annual_lower)
        upper_full <- c(tail(annual_hist, 1), annual_upper)
        
        if (isFALSE(show_all)) {
          xlim_vals    <- c(head(time_fore, 1) - (freq * 2), tail(time_fore, 1))
          hist_in_plot <- annual_hist[time_hist >= xlim_vals[1] & time_hist <= xlim_vals[2]]
          ylim         <- range(c(hist_in_plot, annual_lower, annual_upper), na.rm = TRUE)
          ymin         <- floor(ylim[1] * 2) / 2
          ymax         <- ceiling(ylim[2] * 2) / 2
          yticks       <- seq(ymin, ymax, by = 0.5)
          
          plot.ts(annual_hist, main = paste(colnames(Y)[i], "(annual)"),
                  xlab = "Time", ylab = NULL,
                  xlim = c(head(time_fore, 1) - (freq * 2), tail(time_fore, 1)),
                  ylim = ylim, col = "black", lwd = 2, yaxt = "n")
          abline(v = seq(head(time_hist, 1), tail(time_fore, 1), by = 1 / freq),
                 col = "gray", lty = 2)
          abline(h = yticks, col = "gray", lty = 2)
          axis(side = 2, at = yticks, labels = yticks, las = 1)
          points(as.numeric(time_hist), annual_hist, pch = 16, col = "black")
          points(time_full[-1], m_full[-1], pch = 16, col = "blue")
        } else {
          plot.ts(annual_hist, main = paste(colnames(Y)[i], "(annual)"),
                  xlab = "Time", ylab = NULL, col = "black", lwd = 2,
                  xlim = c(head(time_hist, 1), tail(time_fore, 1)),
                  ylim = range(upper_full, lower_full, annual_hist, na.rm = TRUE))
        }
        polygon(c(time_full, rev(time_full)), c(upper_full, rev(lower_full)),
                col = rgb(0, 0, 1, 0.2), border = NA)
        lines(time_full, m_full, col = "blue", lwd = 2)
      }
      
    } else {
      
      forecast_ret[, i] <- fcst_m
      lower_ret[, i]    <- fcst_lower
      upper_ret[, i]    <- fcst_upper
      
      if (i %in% plot_idx) {
        time_full  <- c(tail(time_hist, 1), time_fore)
        m_full     <- c(tail(smply, 1), fcst_m)
        lower_full <- c(tail(smply, 1), fcst_lower)
        upper_full <- c(tail(smply, 1), fcst_upper)
        
        if (isFALSE(show_all)) {
          xlim_vals    <- c(head(time_fore, 1) - (freq * 2), tail(time_fore, 1))
          hist_in_plot <- smply[time_hist >= xlim_vals[1] & time_hist <= xlim_vals[2]]
          ylim         <- range(c(hist_in_plot, lower_full, upper_full), na.rm = TRUE)
          ymin         <- floor(ylim[1] * 2) / 2
          ymax         <- ceiling(ylim[2] * 2) / 2
          yticks       <- seq(ymin, ymax, by = 0.5)
          
          plot.ts(smply, main = colnames(Y)[i], xlab = "Time", ylab = NULL,
                  xlim = c(head(time_fore, 1) - (freq * 2), tail(time_fore, 1)),
                  ylim = ylim, col = "black", lwd = 2, yaxt = "n")
          abline(v = seq(head(time_hist, 1), tail(time_fore, 1), by = 1 / freq),
                 col = "gray", lty = 2)
          abline(h = yticks, col = "gray", lty = 2)
          axis(side = 2, at = yticks, labels = yticks, las = 1)
          points(as.numeric(time_hist), smply, pch = 16, col = "black")
          points(time_full[-1], m_full[-1], pch = 16, col = "blue")
        } else {
          plot.ts(smply, main = colnames(Y)[i], xlab = "Time", ylab = NULL,
                  col = "black", lwd = 2,
                  xlim = c(head(time_hist, 1), tail(time_fore, 1)),
                  ylim = range(upper_full, lower_full, smply))
        }
        polygon(c(time_full, rev(time_full)), c(upper_full, rev(lower_full)),
                col = rgb(0, 0, 1, 0.2), border = NA)
        lines(time_full, m_full, col = "blue", lwd = 2)
      }
    }
  }
  
  invisible(list(forecast = forecast_ret, lower = lower_ret, upper = upper_ret))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.bvar}
\alias{summary.bvar}
\title{Summarise a fitted steady-state BVAR model}
\usage{
\method{summary}{bvar}(object, pars = NULL, stat = "mean", t = NULL, ...)
}
\arguments{
\item{object}{A steady-state \code{bvar} object that has been passed through
\code{\link{fit}}.}

\item{pars}{Character vector of parameter names to include. If
\code{NULL} (default), all available parameters are displayed.}

\item{stat}{Character. Posterior summary statistic to display:
\code{"mean"} (default) or \code{"median"}.}

\item{t}{Integer. Time index for the innovation covariance matrix if stochastic volatility was estimated.
If \code{NULL}, the latest available \code{t} is used.}

\item{...}{Additional arguments (currently unused).}
}
\value{
Returns the input object invisibly.
}
\description{
Computes and prints posterior summaries from a fitted steady-state \code{bvar} object.
The printed output depends on whether the model is homoscedastic or includes
stochastic volatility (RW or AR1 specification).
}
\details{
The function summarises the following estimated parameters:
\itemize{
\item \code{beta}: \eqn{kp \times k} VAR coefficient matrix
\item \code{Psi}: \eqn{k \times q} steady-state parameter matrix
\item \code{Sigma_u}: innovation covariance matrix (\eqn{k \times k} for homoscedastic,
\eqn{T \times k \times k} for stochastic volatility)
\item If Random Walk stochastic volatility:
\itemize{
\item \code{A}: \eqn{k \times k} lower triangular matrix with ones on the diagonal that describes
the contemporaneous interaction of the endogenous variables
\item \code{phi}: \eqn{k}-dimensional vector of log volatility innovation variances
}
\item If AR1 stochastic volatility:
\itemize{
\item \code{A}: \eqn{k \times k} lower triangular matrix with ones on the diagonal that describes
the contemporaneous interaction of the endogenous variables
\item \code{gamma_0}: \eqn{k}-dimensional vector of log volatility intercepts
\item \code{gamma_1}: \eqn{k}-dimensional vector of log volatility slopes
\item \code{Phi}: \eqn{k \times k} log volatility innovation covariance matrix
}
}
}
\examples{
\donttest{
yt <- matrix(rnorm(20), 10, 2)
bvar_obj <- bvar(data = yt)
bvar_obj <- setup(bvar_obj, p = 1, deterministic = "constant")
bvar_obj <- priors(bvar_obj,
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2))

bvar_obj <- fit(bvar_obj,
                H = 1,
                d_pred = matrix(1),
                iter = 100,
                warmup = 50,
                chains = 1,
                cores = 1)

summary(bvar_obj)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{fit}
\alias{fit}
\title{Estimate the steady-state BVAR model using Stan}
\usage{
fit(x, H = 1, d_pred = NULL, ...)
}
\arguments{
\item{x}{A steady-state \code{bvar} object that has been passed through
\code{\link{setup}} and \code{\link{priors}}.}

\item{H}{Positive Integer. Forecast horizon.
Default is \code{1}.}

\item{d_pred}{Matrix of size \eqn{H \times q}. Future values of the deterministic variables \eqn{d_t}. Default is \code{NULL}, must be provided by the user.}

\item{...}{Additional arguments passed directly to the \code{rstan} function \code{\link[rstan]{sampling}}
(e.g. \code{iter}, \code{warmup}, \code{chains}, \code{cores}, \code{control},
\code{seed}, \code{init}, \code{thin}, \code{algorithm}, \code{pars},
\code{include}, \code{refresh}, \code{verbose}, \code{save_warmup},
\code{sample_file}, \code{diagnostic_file}). If \code{pars}/\code{include} is
used to exclude model parameters required by \code{fit()} for posterior
summaries, an error will be raised.}
}
\value{
A fitted steady-state \code{bvar} object with:
\itemize{
\item \code{x$fit$stan}: An object of class \code{stanfit}
\item \code{x$fit$posterior_means}: List of posterior mean estimates
\item \code{x$fit$posterior_medians}: List of posterior median estimates
}
}
\description{
Estimates the steady-state BVAR model using the No-U-Turn sampler (a variant of Hamiltonian Monte Carlo) via Stan.
Also generates draws from the joint predictive distribution.
Uses the data, setup, and priors stored in the steady-state \code{bvar} object.
}
\details{
The function selects the appropriate precompiled Stan model based on settings from \link{priors}:
\itemize{
\item \code{steady_state_bvar_homoscedastic_jeffreys_prior}
\item \code{steady_state_bvar_homoscedastic_inverse_wishart_prior}
\item \code{steady_state_bvar_RW_stochastic_volatility}
\item \code{steady_state_bvar_AR1_stochastic_volatility}
}
The function estimates the following parameters (see \link{bvar} for details):
\itemize{
\item \code{beta}: \eqn{kp \times k} VAR coefficient matrix
\item \code{Psi}: \eqn{k \times q} steady-state parameter matrix
\item \code{Sigma_u}: innovation covariance matrix (\eqn{k \times k} for homoscedastic,
\eqn{T \times k \times k} for stochastic volatility)
\item If Random Walk stochastic volatility:
\itemize{
\item \code{A}: \eqn{k \times k} lower triangular matrix with ones on the diagonal that describes
the contemporaneous interaction of the endogenous variables
\item \code{phi}: \eqn{k}-dimensional vector of log volatility innovation variances
}
\item If AR1 stochastic volatility:
\itemize{
\item \code{A}: \eqn{k \times k} lower triangular matrix with ones on the diagonal that describes
the contemporaneous interaction of the endogenous variables
\item \code{gamma_0}: \eqn{k}-dimensional vector of log volatility intercepts
\item \code{gamma_1}: \eqn{k}-dimensional vector of log volatility slopes
\item \code{Phi}: \eqn{k \times k} log volatility innovation covariance matrix
}
}
}
\examples{
\donttest{
#homoscedastic with Jeffreys prior, d_t = constant

yt <- matrix(rnorm(20), 10, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p = 1, deterministic = "constant")

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   Jeffreys = TRUE)
                   
H <- 8
d_pred <- matrix(rep(1,8))
colnames(d_pred) <- c("constant")
rownames(d_pred) <- paste("Horizon", 1:H)
print(d_pred)

bvar_obj <- fit(bvar_obj,
                H = H,
                d_pred = d_pred,
                iter = 200,
                warmup = 50,
                chains = 1,
                cores = 1)
                
                
#homoscedastic with inverse-Wishart prior, d_t = constant and dummy

yt <- matrix(rnorm(20), 10, 2)

bvar_obj <- bvar(data = yt)

dummy_variable <- c(rep(1,5), rep(0,5))

bvar_obj <- setup(bvar_obj, p = 1,
                  deterministic = "constant_and_dummy",
                  dummy = dummy_variable)
                  
k <- bvar_obj$setup$k
q <- bvar_obj$setup$q

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, k*q),
                   Omega_Psi = diag(0.1, k*q, k*q),
                   Jeffreys = FALSE) #inverse-Wishart

H <- 8
d_pred <- cbind(rep(1, H), rep(0, H))
colnames(d_pred) <- c("constant", "dummy")
rownames(d_pred) <- paste("Horizon", 1:H)
print(d_pred)

bvar_obj <- fit(bvar_obj,
                H = H,
                d_pred = d_pred,
                iter = 200,
                warmup = 50,
                chains = 1,
                cores = 1)
 

#RW stochastic volatility

yt <- matrix(rnorm(20), 10, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")

k <- bvar_obj$setup$k
n_free_params_A <- bvar_obj$setup$n_free_params_A

SV_priors_RW <- list(
theta_A              =  rep(0, n_free_params_A),
Omega_A              =  diag(1000, n_free_params_A),
mu_log_lambda_1      =  rep(0, k),
sigma2_log_lambda_1  =  rep(1000, k),
alpha_phi            =  rep(5, k),
beta_phi             = (rep(5, k) - 1) * rep(0.1, k)
)

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   SV = TRUE,
                   SV_type = "RW",
                   SV_priors = SV_priors_RW)

bvar_obj <- fit(bvar_obj,
                H = 8,
                d_pred = matrix(rep(1,8)),
                iter = 200,
                warmup = 50,
                chains = 1,
                cores = 1,
                control = list(max_treedepth = 12, adapt_delta = 0.85)
                )


#AR1 stochastic volatility

yt <- matrix(rnorm(20), 10, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")

k <- bvar_obj$setup$k
n_free_params_A <- bvar_obj$setup$n_free_params_A

SV_priors_AR1 <- list(
theta_A               =  rep(0, n_free_params_A),
Omega_A               =  diag(1000, n_free_params_A),
theta_gamma_0         =  rep(0.1, k),
Omega_gamma_0         =  diag(1000, k),
theta_gamma_1         =  rep(0.9, k),
Omega_gamma_1         =  diag(10, k),
theta_log_lambda_1    =  rep(0.1, k)/(1-rep(0.9, k)),
Omega_log_lambda_1    =  diag(1000, k),
V_Phi                 = (10 - k - 1) * diag(k),
m_Phi                 =  10
)

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   SV = TRUE,
                   SV_type = "AR1",
                   SV_priors = SV_priors_AR1)

bvar_obj <- fit(bvar_obj,
                H = 8,
                d_pred = matrix(rep(1,8)),
                iter = 200,
                warmup = 50,
                chains = 1,
                cores = 1,
                control = list(max_treedepth = 12, adapt_delta = 0.85)
                )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stochastic_volatility_plot.R
\name{stochastic_volatility_plot}
\alias{stochastic_volatility_plot}
\title{Plot stochastic volatility estimates and forecasts}
\usage{
stochastic_volatility_plot(
  x,
  ci = 0.95,
  vol = "log_lambda",
  stat = "mean",
  plot_idx = NULL,
  xlim = NULL,
  ylim = NULL
)
}
\arguments{
\item{x}{A steady-state \code{bvar} object that has been passed through
\code{\link{fit}} with stochastic volatility enabled.}

\item{ci}{Numeric. Interval level. Default is \code{0.95}.}

\item{vol}{Character. Volatility representation: \code{"log_lambda"}
(default) or \code{"sd"}.}

\item{stat}{Character. Posterior summary statistic to use as point estimate:
\code{"mean"} (default) or \code{"median"}.}

\item{plot_idx}{Integer vector. Indices of variables to plot. If \code{NULL}
(default), all variables are plotted.}

\item{xlim}{Numeric vector of length 2. Optional x-axis limits.}

\item{ylim}{Numeric vector of length 2. Optional y-axis limits.}
}
\value{
An invisible list with:
\itemize{
\item \code{estimated}: List of posterior summaries for the estimation sample:
\itemize{
\item \code{point}: \eqn{T \times k} matrix of posterior point estimates
(mean or median depending on \code{stat})
\item \code{lower}: \eqn{T \times k} matrix of lower credible bounds
\item \code{upper}: \eqn{T \times k} matrix of upper credible bounds
}
\item \code{predicted}: List of posterior summaries for the forecast horizon:
\itemize{
\item \code{point}: \eqn{(T+H) \times k} matrix of predictive point estimates
(mean or median depending on \code{stat})
\item \code{lower}: \eqn{(T+H) \times k} matrix of lower bound of prediction interval
\item \code{upper}: \eqn{(T+H) \times k} matrix of upper bound of prediction interval
}
}
}
\description{
Produces time series plots of posterior stochastic volatility estimates
over the estimation sample together with predictive paths over the forecast
horizon for a fitted steady-state \code{bvar} object with stochastic volatility.
}
\details{
The function supports two volatility representations:
\itemize{
\item \code{"log_lambda"}: log variances
\item \code{"sd"}: implied standard deviations
}

For each selected variable, the function plots the posterior point estimate
path and credible bands over the estimation sample, and the predictive point
estimate path and prediction intervals over the forecast horizon.
}
\examples{
\donttest{
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p = 1, deterministic = "constant")

k <- bvar_obj$setup$k
n_free_params_A <- bvar_obj$setup$n_free_params_A

SV_priors <- list(
theta_A             =  rep(0, n_free_params_A),
Omega_A             =  diag(1000, n_free_params_A),
mu_log_lambda_1     =  rep(0, k),
sigma2_log_lambda_1 =  rep(1000, k),
alpha_phi           =  rep(5, k),
beta_phi            = (rep(5, k) - 1) * rep(0.1, k)
)

bvar_obj <- priors(bvar_obj,
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   SV = TRUE,
                   SV_type = "RW",
                   SV_priors = SV_priors)

bvar_obj <- fit(bvar_obj,
                H = 8,
                d_pred = matrix(rep(1, 8)),
                iter = 200,
                warmup = 50,
                chains = 1,
                cores = 1,
                verbose = FALSE)

stochastic_volatility_plot(bvar_obj, ci = 0.95)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{priors}
\alias{priors}
\title{Specify priors for the steady-state BVAR model}
\usage{
priors(
  x,
  lambda_1 = 0.2,
  lambda_2 = 0.5,
  lambda_3 = 1,
  first_own_lag_prior_mean = NULL,
  theta_Psi = NULL,
  Omega_Psi = NULL,
  Jeffreys = TRUE,
  SV = FALSE,
  SV_type = NULL,
  SV_priors = NULL
)
}
\arguments{
\item{x}{A steady-state \code{bvar} object that has been passed through \code{\link{setup}}.}

\item{lambda_1}{Numeric. Overall tightness of the Minnesota prior.
Default \code{0.2}.}

\item{lambda_2}{Numeric. Cross-equation tightness of the Minnesota prior. Default \code{0.5}.}

\item{lambda_3}{Numeric. Lag decay rate of the Minnesota prior. Default \code{1}.}

\item{first_own_lag_prior_mean}{Numeric vector of length \code{k}. Prior means for the first own lags
of the variables. If \code{NULL} (default), a zero vector is used.}

\item{theta_Psi}{Numeric vector. Prior mean vector for \eqn{\text{vec}(\Psi)}, i.e. the steady-state parameters. If \code{NULL} (default),
the OLS estimates are used.}

\item{Omega_Psi}{Numeric matrix. Prior covariance matrix for \eqn{\text{vec}(\Psi)}, i.e. the steady-state parameters. If \code{NULL} (default),
a diagonal matrix with variances \code{1000} is used.}

\item{Jeffreys}{Logical. If \code{TRUE} (default), uses Jeffreys prior for the innovation covariance matrix.
If \code{FALSE}, uses an uninformative inverse-Wishart prior. Only considered if \code{SV=FALSE}.}

\item{SV}{Logical. If \code{TRUE}, enables stochastic volatility specification.
Default \code{FALSE}.}

\item{SV_type}{Character. Type of stochastic volatility model. Must be either \code{"RW"} or \code{"AR1"}.
Required if \code{SV = TRUE}.}

\item{SV_priors}{List. User-supplied stochastic volatility priors. Required when \code{SV = TRUE}.
The list must contain the following named elements depending on \code{SV_type}:
\itemize{
\item For \code{"RW"}: \code{theta_A}, \code{Omega_A}, \code{mu_log_lambda_1},
\code{sigma2_log_lambda_1}, \code{alpha_phi}, \code{beta_phi}.
\item For \code{"AR1"}: \code{theta_A}, \code{Omega_A}, \code{theta_gamma_0},
\code{Omega_gamma_0}, \code{theta_gamma_1}, \code{Omega_gamma_1},
\code{theta_log_lambda_1}, \code{Omega_log_lambda_1}, \code{V_Phi}, \code{m_Phi}.
}}
}
\value{
The steady-state \code{bvar} object with an appended \code{priors} list containing:
\item{theta_beta}{Prior mean vector for \eqn{\text{vec}(\beta)} constructed with the Minnesota prior}
\item{Omega_beta}{Prior covariance matrix for \eqn{\text{vec}(\beta)} constructed with the Minnesota prior}
\item{theta_Psi}{Prior mean vector for \eqn{\text{vec}(\Psi)}, i.e. the steady-state parameters}
\item{Omega_Psi}{Prior covariance matrix for \eqn{\text{vec}(\Psi)}, i.e. the steady-state parameters}
\item{Jeffreys}{Indicator for Jeffreys prior usage}
\item{Sigma_AR}{Residual variance estimates from univariate AR fits, which are used by the Minnesota prior}
\item{m}{Inverse-Wishart prior degrees of freedom (if \code{Jeffreys = FALSE})}
\item{V}{Inverse-Wishart prior scale matrix (if \code{Jeffreys = FALSE})}
\item{SV}{Logical indicator for stochastic volatility specification}
\item{SV_type}{Stochastic volatility specification type}
\item{SV_priors}{User-supplied SV prior list (if \code{SV = TRUE})}
}
\description{
This function prepares the priors. The Minnesota prior is used for the autoregressive parameters, and is determined by
the overall tightness, cross-equation tightness, and the lag decay rate.
For the steady-state parameters, a normal prior is used. For the covariance matrix of the innovations,
the user can choose between Jeffreys prior or an uninformative inverse-Wishart prior.
Optionally enables stochastic volatility where the covariance matrix varies over time.
}
\details{
The goal is to estimate \eqn{\beta, \Psi}, and \eqn{\Sigma_u}, so priors are needed.
Following Villani (2009), prior independence between \eqn{\beta, \Psi} and \eqn{\Sigma_u} is assumed. For \eqn{\beta}, i.e. the autoregressive parameter matrix,
the Minnesota prior is used

\deqn{\mathrm{vec}(\beta) \sim \mathrm{N}_{kpk} \left[\theta_\beta,\Omega_\beta\right]}

The prior means (the elements of \eqn{\theta_\beta}) are set to

\deqn{
\begin{aligned}
\mathrm{E}\left(\Pi_{\ell}^{(i,j)}\right)&=
\begin{cases}\kappa & \text{if } \ell = 1 \ \text{and} \ i = j \\0 & \text{otherwise}\end{cases}\\
\kappa&=\begin{cases}\kappa^{level} & \text{if } \text{variable} \ i \ \text{is in level} \\
\kappa^{\Delta} & \text{if } \text{variable} \ i \ \text{is in difference}
\end{cases}\\
\end{aligned}
}

Here, the autoregressive coefficient \eqn{\Pi_{\ell}^{(i,j)}} is element \eqn{\left(i,j\right)}
of \eqn{\Pi_{\ell}} for \eqn{\ell=1,\dots,p}. As such, the Minnesota prior sets all prior means
for the elements in \eqn{\beta} to \eqn{0}, except for the elements that relate to the first
own lags of the variables, which are set to \eqn{\kappa}. If variable \eqn{i} is in level
(e.g. nominal interest rate), then \eqn{\kappa=\kappa^{level}}, and typical choices for
\eqn{\kappa^{level}} are \eqn{1} or \eqn{0.9}. Evaluating the equations at their prior means,
equation \eqn{i} becomes a random walk if \eqn{\kappa^{level}=1} and a persistent stationary
AR(1) process if \eqn{\kappa^{level}=0.9}. Since the steady state only exists if the process
is stationary, \eqn{0.9} is recommended for the steady-state BVAR. If variable \eqn{i} is in difference
(e.g. output growth), then \eqn{\kappa=\kappa^{\Delta}}, and the most common choice for
\eqn{\kappa^{\Delta}} is \eqn{0}, i.e. equation \eqn{i} becomes (when evaluating it at its prior means)
a random walk expressed in first differences. If a differenced variable still shows some degree
of persistence (can be examined with an ACF plot), a suitable value for \eqn{\kappa^{\Delta}} can
be (for example) \eqn{0.6} instead of \eqn{0}. Moving on to the prior variances, \eqn{\Omega_\beta} is a
diagonal matrix containing the prior variances for the elements in \eqn{\beta}. They are specified as

\deqn{\mathrm{Var}\left(\Pi_{\ell}^{(i,j)}\right)=
\begin{cases}\left(\frac{\lambda_1}{\ell^{\lambda_3}}\right)^2 & \text{if } i = j \\
\left(\frac{\lambda_1 \lambda_2\sigma_i}{\ell^{\lambda_3}\sigma_j}\right)^2& \text{if } i \neq j
\end{cases}}

Here \eqn{\lambda_1}, \eqn{\lambda_2}, and \eqn{\lambda_3} are scalar hyperparameters known as
the overall tightness, the cross-equation tightness and the lag decay rate. Furthermore,
\eqn{\sigma_i^2} is the \eqn{(i,i)}:th element of \eqn{\Sigma_u}, which is unknown and therefore
replaced with an estimate. In this package, it is replaced by the least squares residual variance
from a univariate autoregression for variable \eqn{i} with \eqn{p} lags
(including the constant and dummy/trend variable if applicable). Moving on to \eqn{\Psi}, the steady-state parameter matrix, the
prior is

\deqn{\mathrm{vec}(\Psi) \sim \mathrm{N}_{kq}\left[\theta_\Psi,\Omega_\Psi\right]}

This is the core of the steady-state BVAR model.
In \eqn{\theta_\Psi}, one specifies the prior beliefs about the location of the steady state,
and in \eqn{\Omega_\Psi}, which is assumed to be a diagonal matrix, one specifies the degree
of certainty in those prior beliefs. The prior for \eqn{\Sigma_u} is either the usual non-informative Jeffreys prior

\deqn{p(\Sigma_u) \propto\left|\Sigma_u \right|^{-(k+1)/2}}

or a proper uninformative inverse-Wishart prior

\deqn{\Sigma_u \sim \mathrm{IW}(V,m)}

where \eqn{V} is the scale matrix and \eqn{m} is the number of degrees of freedom.
An uninformative prior is specified by setting
\eqn{V=(m-k-1)\hat{\Sigma}_u} where \eqn{\hat{\Sigma}_u} is the least squares estimate
from the VAR(\eqn{p}) (including the constant and dummy/trend variable if applicable), and \eqn{m=k+2}.
For the stochastic volatility specifications, the innovation covariance matrix is now time-varying \eqn{\Sigma_{u,t}}.
Therefore, stochastic volatility priors are needed, see \link{bvar} for more details. For the Random Walk
(\code{"RW"}) stochastic volatility specification, the following priors are used

\deqn{\begin{aligned}a &\sim \mathrm{N}(\theta_A, \Omega_A) \\
\ln \lambda_{i,1} &\sim \mathrm{N}(\mu_{\ln \lambda_{i,1}}, \sigma^2_{\ln \lambda_{i,1}}) \\
\phi_i &\sim \mathrm{IG}(\alpha_{\phi_i},\beta_{\phi_i})\end{aligned}}

Here \eqn{a} is a \eqn{k(k-1)/2}-dimensional vector that collects the free parameters in \eqn{A} in row-major order,
and \eqn{\ln \lambda_{i,1}} are the time \eqn{t=1} values (initial conditions) of \eqn{\ln \lambda_{i,t}} for \eqn{i=1,\dots,k}.
Furthermore, \eqn{\phi_i} for \eqn{i=1,\dots,k} are the log volatility innovation variances. For the AR(1) (\code{"AR1"}) stochastic volatility specification, the following priors are used

\deqn{\begin{aligned}a &\sim \mathrm{N}(\theta_A, \Omega_A) \\
\gamma_{0} &\sim \mathrm{N}(\theta_{\gamma_0}, \Omega_{\gamma_0}) \\
\gamma_{1} &\sim \mathrm{N}(\theta_{\gamma_1}, \Omega_{\gamma_1}) \\
\ln \lambda_{1} &\sim \mathrm{N}(\theta_{\ln \lambda_{1}}, \Omega_{\ln \lambda_{1}}) \\
\Phi &\sim \mathrm{IW}(V_{\Phi},m_{\Phi})\end{aligned}}

Here \eqn{a} is again the \eqn{k(k-1)/2}-dimensional vector that collects the free parameters in \eqn{A} in row-major order,
and \eqn{\ln \lambda_1} is a \eqn{k}-dimensional vector containing the time \eqn{t=1} values (initial conditions) of \eqn{\ln \lambda_{t}}.
Furthermore, \eqn{\gamma_{0}} is a \eqn{k}-dimensional vector of log volatility intercepts, \eqn{\gamma_{1}} is a \eqn{k}-dimensional vector of log volatility
slopes, and \eqn{\Phi} is the \eqn{k \times k} log volatility innovation covariance matrix.

For details on the homoscedastic steady-state BVAR model, see Villani (2009).
For details on the Random Walk stochastic volatility steady-state BVAR model, see Clark (2011).
See Carriero, Clark, and Marcellino (2024) for the AR(1) stochastic volatility
specification applied to a conventional BVAR.
}
\examples{
#homoscedastic with Jeffreys prior
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=1)

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   Jeffreys = TRUE,
                   SV = FALSE,
                   SV_type = NULL,
                   SV_priors = NULL)
                   
#RW stochastic volatility
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=1)

k <- bvar_obj$setup$k
n_free_params_A <- bvar_obj$setup$n_free_params_A

SV_priors_RW <- list(
theta_A              =  rep(0, n_free_params_A),
Omega_A              =  diag(1000, n_free_params_A),
mu_log_lambda_1      =  rep(0, k),
sigma2_log_lambda_1  =  rep(1000, k),
alpha_phi            =  rep(5, k),
beta_phi             = (rep(5, k) - 1) * rep(0.1, k)
)

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   SV = TRUE,
                   SV_type = "RW",
                   SV_priors = SV_priors_RW)
                   
#AR1 stochastic volatility
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=1)

k <- bvar_obj$setup$k
n_free_params_A <- bvar_obj$setup$n_free_params_A

SV_priors_AR1 <- list(
theta_A               =  rep(0, n_free_params_A),
Omega_A               =  diag(1000, n_free_params_A),
theta_gamma_0         =  rep(0.1, k),
Omega_gamma_0         =  diag(1000, k),
theta_gamma_1         =  rep(0.9, k),
Omega_gamma_1         =  diag(10, k),
theta_log_lambda_1    =  rep(0.1, k)/(1-rep(0.9, k)),
Omega_log_lambda_1    =  diag(1000, k),
V_Phi                 = (10 - k - 1) * diag(k),
m_Phi                 =  10
)

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   SV = TRUE,
                   SV_type = "AR1",
                   SV_priors = SV_priors_AR1)

}
\references{
Carriero, A., Clark, T. E., and Marcellino, M. (2024).
Capturing macro-economic tail risks with Bayesian vector autoregressions.
\emph{Journal of Money, Credit and Banking}, 56(5), pp. 1099–1127.

Clark, T. E. (2011). Real-time density forecasts from Bayesian vector autoregressions
with stochastic volatility. \emph{Journal of Business & Economic Statistics}, 29(3), pp. 327–341.

Villani, M. (2009). Steady-state priors for vector autoregressions.
\emph{Journal of Applied Econometrics}, 24(4), pp. 630–650.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppi.R
\name{ppi}
\alias{ppi}
\title{Prior Probability Interval for a Normal Distribution}
\usage{
ppi(l, u, interval = 0.95, annualized_growthrate = FALSE, freq = 4)
}
\arguments{
\item{l}{Numeric. The lower bound of the prior probability interval.}

\item{u}{Numeric. The upper bound of the prior probability interval.}

\item{interval}{Numeric. The prior probability mass within the interval.
Default \code{0.95}, i.e. 95\%.}

\item{annualized_growthrate}{Logical. If \code{TRUE}, treats \code{l} and \code{u} as
bounds on the annualized steady-state growth rate and calculates the implied mean
and variance on the corresponding quarterly/monthly scale. Useful if you are working with
a variable \code{diff(log(x))} (which may be scaled by 100). Default \code{FALSE}.}

\item{freq}{Integer. The data frequency (e.g. \code{4} for quarterly).
Only used when \code{annualized_growthrate = TRUE}. Default \code{4}.}
}
\value{
A list with two elements: \code{mean} and \code{var}, giving the
mean and variance of the implied normal distribution.
}
\description{
Calculates the mean and variance of a normal prior probability interval.
Given a lower and upper bound of a prior probability interval
this function recovers the implied normal prior parameters.
Useful for specifying informative priors
on the steady-state parameters (elements of \eqn{\Psi}).
}
\details{
Consider a CPI variable \code{CPI <- data$CPI}, observed at quarterly frequency.
In the model, quarter-on-quarter inflation is used \code{x <- 100*diff(log(CPI))}.
Suppose the prior belief is that annualized steady-state inflation lies between 1.7 and 2.3
with 95\% probability (mean 2). On the quarterly scale used by \code{x}, this corresponds to a
95\% interval of 0.425 to 0.575 (mean 0.5). Since it is typically more natural to elicit a prior
on the annualized scale, the \code{annualized_growthrate} argument performs this conversion.
See \code{vignette("Homoscedastic-steady-state-BVAR")} for usage in practice.
}
\examples{
ppi(l = 1.7, u = 2.3, interval = 0.95)
ppi(l = 1.7, u = 2.3, interval = 0.95, annualized_growthrate = TRUE, freq = 4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/irf.R
\name{IRF}
\alias{IRF}
\title{Impulse Response Functions for a fitted steady-state BVAR model}
\usage{
IRF(
  x,
  H = 16,
  response = NULL,
  shock = NULL,
  type = c("median", "mean"),
  method = c("OIRF", "GIRF"),
  ci = 0.95,
  t = NULL,
  growth_rate_idx = NULL
)
}
\arguments{
\item{x}{A steady-state \code{bvar} object that has been passed through \code{\link{fit}}.}

\item{H}{Integer. The forecast horizon for the IRF. Default \code{16}.}

\item{response}{Integer. Index of the response variable to plot. If
\code{NULL} (default), all responses are plotted.}

\item{shock}{Integer. Index of the shock variable to plot. If \code{NULL}
(default), all shocks are plotted.}

\item{type}{Character. Whether to use \code{"median"} or \code{"mean"} as
the point estimate. Default \code{"median"}.}

\item{method}{Character. The IRF method: \code{"OIRF"} for orthogonalized
or \code{"GIRF"} for generalized impulse responses. Default \code{"OIRF"}.}

\item{ci}{Numeric. The credible interval width. Default \code{0.95}, i.e. 95\%.}

\item{t}{Integer. Time index for the covariance matrix when using stochastic
volatility models. If \code{NULL} (default), the last time \code{t} is used.}

\item{growth_rate_idx}{Integer vector. Indices of variables for which the impulse
response is converted from a quarterly or monthly log first difference to an
annual growth rate response, i.e. \eqn{\ln x_{t} - \ln x_{t-f}}, where \eqn{f} is
the frequency of the data (4 for quarterly, 12 for monthly).
Only suitable for variables specified as \eqn{\ln x_{t} - \ln x_{t-1}}, i.e.
\code{diff(log(x))} or \code{100*diff(log(x))}. Computed by summing up to
\eqn{f} periods of the impulse response, treating the response in periods
prior to the shock as zero. Default is \code{NULL}.}
}
\value{
Invisibly returns a list with three arrays: the point estimate IRF, \code{lower}, and
\code{upper} credible bounds, each of dimension \code{k x k x (H+1)}.
}
\description{
Computes and plots impulse response functions (IRFs) from a fitted
steady-state \code{bvar} object. Supports both orthogonalized (OIRF) and generalized
(GIRF) impulse responses, with optional conversion to annual growth rates.
}
\examples{
\donttest{
#homoscedastic with Jeffreys prior
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   Jeffreys = TRUE,
                   SV = FALSE,
                   SV_type = NULL,
                   SV_priors = NULL)
                   
bvar_obj <- fit(bvar_obj,
                H = 8,
                d_pred = matrix(rep(1,8)),
                iter = 200,
                warmup = 50,
                chains = 1,
                cores = 1)
                
(IRF(bvar_obj))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Villani2009}
\alias{Villani2009}
\title{Villani (2009) dataset}
\format{
\subsection{\code{Villani2009}}{

A multivariate time series object with 104 rows and 7 columns:
\describe{
\item{delta y_f}{foreign GDP growth}
\item{pi_f}{foreign CPI inflation}
\item{i_f}{foreign 3-month interest rate}
\item{delta y}{domestic GDP growth}
\item{pi}{domestic CPI inflation}
\item{i}{domestic 3-month interest rate}
\item{q}{level of the real exchange rate defined as \eqn{q = s + p_f - p}, where \eqn{p_f} and \eqn{p}
are the foreign and domestic CPI levels in logs and \eqn{s} is the log of the trade-weighted nominal exchange rate}
}
}
}
\source{
Villani, M. (2009). Steady-state priors for vector autoregressions.
\emph{Journal of Applied Econometrics}, 24(4), pp. 630-650.
}
\usage{
Villani2009
}
\description{
Swedish macroeconomic data (1980Q1–2005Q4) from Section 4.1 in Villani (2009)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SteadyStateBVAR-package.R
\docType{package}
\name{SteadyStateBVAR-package}
\alias{SteadyStateBVAR}
\alias{SteadyStateBVAR-package}
\title{SteadyStateBVAR: Bayesian Vector Autoregressions with Steady-State Priors}
\description{
Provides estimation of Bayesian vector autoregression (BVAR) models with steady-state priors via 'Stan', along with functions for unconditional and conditional forecasting, as well as impulse response analysis. For details on the steady-state BVAR model see Villani (2009) \doi{10.1002/jae.1065}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/markjwbecker/SteadyStateBVAR}
  \item \url{https://markjwbecker.github.io/SteadyStateBVAR/}
  \item Report bugs at \url{https://github.com/markjwbecker/SteadyStateBVAR/issues}
}

}
\author{
\strong{Maintainer}: Mark Becker \email{mark.jw.becker@gmail.com} [copyright holder]

Authors:
\itemize{
  \item Mark Becker \email{mark.jw.becker@gmail.com} [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bvar.R
\name{bvar}
\alias{bvar}
\title{Create a steady-state BVAR model object}
\usage{
bvar(data)
}
\arguments{
\item{data}{A numeric matrix or time series of data where each column is a
variable and each row is a time period.}
}
\value{
An object of class \code{bvar}.
}
\description{
Initialises a steady-state \code{bvar} object. This is the starting point for all models in \code{SteadyStateBVAR}.
After creation, pass the object sequentially to \code{\link{setup}}, \code{\link{priors}},
and \code{\link{fit}} to build and estimate the model.
}
\details{
The model takes the form

\deqn{y_t = \Psi d_t + \Pi_1(y_{t-1}-\Psi d_{t-1})+\dots+\Pi_p(y_{t-p}-\Psi d_{t-p})+u_t}

where \eqn{y_t} is an \eqn{k}-dimensional vector of endogenous variables at time \eqn{t}, and \eqn{d_t} is
a \eqn{q}-dimensional vector of deterministic (exogenous) variables at time \eqn{t}.
Here \eqn{\Pi_\ell} for \eqn{\ell=1,\dots,p} is a \eqn{(k \times k)} matrix of autoregressive parameters,
and \eqn{\Psi} is a \eqn{(k \times q)} matrix of steady-state parameters. Note that
\eqn{\mathrm{E}(y_t)=\mu_t=\Psi d_t} is the unconditional mean, or the \strong{steady state} of the process.
One can stack the (transposed) \eqn{\Pi_i} matrices in the \eqn{(kp \times k)} matrix \eqn{\beta}
\deqn{\beta=\begin{bmatrix}\Pi'_1 \\ \vdots  \\\Pi'_p\end{bmatrix}}
Then the model can be rewritten as a nonlinear regression (Karlsson, 2013)
\deqn{y_t' =d_t'\Psi' + \left[w_t'-q_t'(I_p \otimes \Psi') \right]\beta +u_t'}
where where \eqn{w_t'=(y_{t-1}',\dots,y_{t-p}')} is a \eqn{kp}-dimensional vector of lagged endogenous variables
and \eqn{q_t'=(d_{t-1}',\dots,d_{t-p}')} is a \eqn{qp}-dimensional vector of lagged deterministic (exogenous) variables,
\eqn{I_p} is the \eqn{(p \times p)} identity matrix and \eqn{\otimes} denotes the Kronecker product.
This is how the likelihood is written in the Stan code. The goal is to estimate \eqn{\beta, \Psi}, and \eqn{\Sigma_u}.

For the innovations to the model, in the case of the homoscedastic steady-state BVAR, they are \eqn{u_t \overset{\text{iid}}{\sim} \mathrm{N_k}(0,\Sigma_u)}.
However, for models with stochastic volatility, there is instead a time-varying covariance matrix \eqn{u_t \sim \mathrm{N_k}(0,\Sigma_{u,t})}.
The innovations then take the form

\deqn{\begin{aligned} u_t &= A^{-1} \Lambda^{0.5}_t \epsilon_t \\
\epsilon_t &\overset{\text{iid}}{\sim} \mathrm{N}(0, \mathrm{I}_k)\end{aligned}}

where \eqn{A} is a lower triangular matrix with ones on the diagonal that describes
the contemporaneous interaction of the endogenous variables, and

\deqn{\Lambda_t = \mathrm{diag}(\lambda_{1,t},\dots,\lambda_{k,t})}

contains the time-varying volatilities.
For the \code{AR1} stochastic volatility specification, the log volatilities follow AR(1) processes

\deqn{\ln \lambda_{i,t} = \gamma_{0,i} + \gamma_{1,i} \ln \lambda_{i,t-1} + \nu_{i,t}, \ i=1,\dots,k}

where the log volatility AR(1) processes are restricted to the stationary region, i.e. \eqn{|\gamma_{1,i}|<1 \ \forall i}.
For the \code{RW} stochastic volatility specification, the log volatilities follow Random Walk processes

\deqn{\gamma_{0,i}=0, \ \gamma_{1,i}=1 \ \forall i}

The innovations to the log volatilities follow in the \code{AR1} case

\deqn{\nu_{t} = (\nu_{1,t},\dots,\nu_{k,t})'\overset{\text{iid}}{\sim} \mathrm{N}(0, \Phi)}

where \eqn{\Phi} \emph{is not diagonal} and as such the innovations to the log
volatilities are allowed to be correlated across variables. For the \code{RW} case, \eqn{\Phi} \emph{is diagonal}
with variances \eqn{\phi_i} for \eqn{i=1,\dots,k}.

Note that under both stochastic volatility specifications, the time-varying covariance matrix is

\deqn{\Sigma_{u,t} = A^{-1} \Lambda_t (A^{-1})'}

For details on the homoscedastic steady-state BVAR model, see Villani (2009).
For details on the Random Walk stochastic volatility steady-state BVAR model, see Clark (2011).
See Carriero, Clark, and Marcellino (2024) for the above-mentioned AR(1) stochastic volatility
specification applied to a conventional BVAR.
}
\examples{
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)
}
\references{
Carriero, A., Clark, T. E., and Marcellino, M. (2024).
Capturing macro-economic tail risks with Bayesian vector autoregressions.
\emph{Journal of Money, Credit and Banking}, 56(5), pp. 1099–1127.

Clark, T. E. (2011). Real-time density forecasts from Bayesian vector autoregressions
with stochastic volatility. \emph{Journal of Business & Economic Statistics}, 29(3), pp. 327–341.

Karlsson, S. (2013). Forecasting with Bayesian vector autoregression.
In: Elliott, G. and Timmermann, A. (eds), \emph{Handbook of Economic Forecasting}.
Elsevier B.V., Vol. 2, Part B, pp. 791–897.

Villani, M. (2009). Steady-state priors for vector autoregressions.
\emph{Journal of Applied Econometrics}, 24(4), pp. 630–650.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup.R
\name{setup}
\alias{setup}
\title{Set up the steady-state BVAR model}
\usage{
setup(
  x,
  p = 1,
  deterministic = c("constant", "constant_and_dummy", "constant_and_trend"),
  dummy = NULL
)
}
\arguments{
\item{x}{A steady-state \code{bvar} object created by \code{\link{bvar}}.}

\item{p}{Integer. The lag order of the VAR. Default \code{1}.}

\item{deterministic}{Character. The deterministic component to include.
One of \code{"constant"} (default), \code{"constant_and_dummy"}, or
\code{"constant_and_trend"}.}

\item{dummy}{Numeric vector of a dummy variable. Only
used when \code{deterministic = "constant_and_dummy"}. Default \code{NULL}.}
}
\value{
The steady-state \code{bvar} object with a \code{setup} list containing the
required components for prior specification and estimation, and also the OLS estimates.
}
\description{
Prepares the components needed for prior specification and estimation.
Also computes OLS estimates.
}
\examples{
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forecast.R
\name{forecast}
\alias{forecast}
\title{Forecast from a fitted steady-state BVAR model}
\usage{
forecast(
  x,
  pi = 0.95,
  fcst_type = c("mean", "median"),
  growth_rate_idx = NULL,
  plot_idx = NULL,
  show_all = FALSE
)
}
\arguments{
\item{x}{A steady-state \code{bvar} object that has been passed through \code{\link{fit}}.}

\item{pi}{Numeric. The prediction interval width. Default \code{0.95}, i.e. 95\% prediction interval.}

\item{fcst_type}{Character. Whether to use \code{"mean"} or \code{"median"}
as the point forecast. Default \code{"mean"}.}

\item{growth_rate_idx}{Integer vector. Indices of variables of which to convert forecasts to
annual growth rates \eqn{\ln x_{t} - \ln x_{t-f}}, where \eqn{f} is
the frequency of the data (4 for quarterly, 12 for monthly).
Only suitable for variables specified as \eqn{\ln x_{t} - \ln x_{t-1}}, i.e.
\code{diff(log(x))} or \code{100*diff(log(x))}.
Computed by summing up to \eqn{f} log first differences. Default is \code{NULL}.}

\item{plot_idx}{Integer vector. Indices of variables to plot. If \code{NULL}
(default), all variables are plotted. Forecasts are always computed and
returned for all variables, regardless of \code{plot_idx}.}

\item{show_all}{Logical. If \code{FALSE} (default), only the last two years
of history are shown alongside the forecast. If \code{TRUE}, the full
history is shown.}
}
\value{
Invisibly returns a list with three matrices: \code{forecast}, \code{lower}, and
\code{upper}, each of dimension \code{H x k} where \code{H} is the
forecast horizon and \code{k} is the number of variables.
}
\description{
Computes and plots forecasts from a fitted \code{bvar} object. Draws from the
joint predictive distribution are used to construct point forecasts and
prediction intervals. Optionally converts monthly or quarterly growth rate forecasts to annual growth
rates for selected variables.
}
\examples{
\donttest{
#homoscedastic with Jeffreys prior
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   Jeffreys = TRUE,
                   SV = FALSE,
                   SV_type = NULL,
                   SV_priors = NULL)
                   
bvar_obj <- fit(bvar_obj,
                H = 8,
                d_pred = matrix(rep(1,8)),
                iter = 200,
                warmup = 50,
                chains = 1,
                cores = 1)

(fcst <- forecast(bvar_obj, pi = 0.90, show_all = TRUE))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/restrict_beta.R
\name{restrict_beta}
\alias{restrict_beta}
\title{Restrict VAR coefficients to zero}
\usage{
restrict_beta(x, restriction_matrix)
}
\arguments{
\item{x}{A steady-state \code{bvar} object that has been passed through \code{\link{priors}}.}

\item{restriction_matrix}{A numeric matrix of dimension \code{k*p x k} where
entries of \code{0} indicate coefficients to be restricted to zero and
entries of \code{1} indicate unrestricted coefficients.}
}
\value{
The steady-state \code{bvar} object with the restriction matrix stored in
\code{setup} and the prior covariance matrix \code{Omega_beta} updated accordingly.
}
\description{
Applies zero restrictions to the VAR coefficient matrix \eqn{\beta} by setting the
corresponding prior variances in \code{Omega_beta} to a value near zero (prior means are zero by default in the Minnesota prior).
}
\details{
The steady-state BVAR model takes the form

\deqn{y_t = \Psi d_t + \Pi_1(y_{t-1}-\Psi d_{t-1})+\dots+\Pi_p(y_{t-p}-\Psi d_{t-p})+u_t}

where \eqn{y_t} is an \eqn{k}-dimensional vector of endogenous variables at time \eqn{t}, and \eqn{d_t} is
a \eqn{q}-dimensional vector of deterministic (exogenous) variables at time \eqn{t}.
Here \eqn{\Pi_\ell} for \eqn{\ell=1,\dots,p} is a \eqn{(k \times k)} matrix of autoregressive parameters,
and \eqn{\Psi} is a \eqn{(k \times q)} matrix of steady-state parameters.
One can stack the (transposed) \eqn{\Pi_i} matrices in the \eqn{(kp \times k)} matrix \eqn{\beta}
\deqn{\beta=\begin{bmatrix}\Pi'_1 \\ \vdots  \\\Pi'_p\end{bmatrix}}

This function puts zero restrictions on the elements of \eqn{\beta}.
}
\examples{
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=2)

bvar_obj <- priors(bvar_obj,
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2))
                   
p <- bvar_obj$setup$p
k <- bvar_obj$setup$k

restriction_matrix <- matrix(1,k*p,k, dimnames=list(NULL,c("y1","y2")))

#restrict beta so y2 does not granger-cause y1

restriction_matrix[2, 1] <- 0
restriction_matrix[4, 1] <- 0

print(restriction_matrix)

bvar_obj <- restrict_beta(bvar_obj, restriction_matrix)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_forecast.R
\name{conditional_forecast}
\alias{conditional_forecast}
\title{Conditional forecasts from a fitted steady-state BVAR model}
\usage{
conditional_forecast(
  bvar_obj,
  conditions,
  pi = 0.95,
  fcst_type = c("mean", "median"),
  growth_rate_idx = NULL,
  plot_idx = NULL
)
}
\arguments{
\item{bvar_obj}{A steady-state \code{bvar} object that has been passed through
\code{\link{fit}}.}

\item{conditions}{A data frame with three columns: \code{var} (integer index
of the conditioned variable), \code{horizon} (integer forecast horizon at
which the condition is imposed), and \code{value} (numeric, the imposed
condition). Note that \code{var} and \code{horizon} must be integers, as
they are used for indexing.}

\item{pi}{Numeric. The prediction interval width. Default \code{0.95}, i.e. 95\% prediction interval.}

\item{fcst_type}{Character. Whether to use \code{"mean"} or \code{"median"}
as the point forecast. Default \code{"mean"}.}

\item{growth_rate_idx}{Integer vector. Indices of variables of which to convert forecasts to
annual growth rates \eqn{\ln x_{t} - \ln x_{t-f}}, where \eqn{f} is
the frequency of the data (4 for quarterly, 12 for monthly).
Only suitable for variables specified as \eqn{\ln x_{t} - \ln x_{t-1}}, i.e.
\code{diff(log(x))} or \code{100*diff(log(x))}.
Computed by summing up to \eqn{f} log first differences. Default is \code{NULL}.}

\item{plot_idx}{Integer vector. Indices of variables to plot. If \code{NULL}
(default), all variables are plotted. Forecasts are always computed and
returned for all variables, regardless of \code{plot_idx}.}
}
\value{
Invisibly returns a list with three matrices: \code{forecast}, \code{lower}, and
\code{upper}, each of dimension \code{H x k}.
}
\description{
Computes and plots conditional forecasts from a fitted steady-state \code{bvar} object.
Conditions are imposed on specific variables at specific horizons using Algorithm 3.3.1
from Dieppe, Legrand, and van Roye (2016). Both conditional and unconditional
forecasts are plotted for comparison. Please note that for the moment, conditional forecasting
is only enabled for the homoscedastic steady-state BVAR, i.e. when \code{SV=FALSE} in \code{priors()}.
}
\details{
See Section 5.3 of Dieppe, Legrand, and van Roye (2016) for more details.
Please note the limitations of this method, see the detailed discussion in Section 5.4 of Dieppe, Legrand, and van Roye (2016).
}
\examples{
\donttest{
#homoscedastic with Jeffreys prior
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj, p=1, deterministic = "constant")

bvar_obj <- priors(bvar_obj,
                   lambda_1 = 0.2,
                   lambda_2 = 0.5,
                   lambda_3 = 1,
                   first_own_lag_prior_mean = rep(1,2),
                   theta_Psi = rep(0, 2),
                   Omega_Psi = diag(0.1, 2, 2),
                   Jeffreys = TRUE,
                   SV = FALSE,
                   SV_type = NULL,
                   SV_priors = NULL)
                   
bvar_obj <- fit(bvar_obj,
                H = 8,
                d_pred = matrix(rep(1,8)),
                iter = 200,
                warmup = 50,
                chains = 1,
                cores = 1)
                
conditions <- data.frame(var = rep(2,8),
                         horizon = rep(1:8),
                         value   = rep(1,8))
                         
(cond_fcst <- conditional_forecast(bvar_obj,
                                  conditions,
                                  pi=0.68,
                                  fcst_type = "mean"))
}
}
\references{
Dieppe, A., Legrand, R., and van Roye, B. (2016).
The BEAR toolbox. Working Paper Series, No. 1934. European Central Bank.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{KoopKorobilis2010}
\alias{KoopKorobilis2010}
\title{Koop and Korobilis (2010) dataset}
\format{
\subsection{\code{KoopKorobilis2010}}{

A multivariate time series object with 215 rows and 3 columns:
\describe{
\item{delta pi}{inflation rate (annual percentage change in a chain-weighted GDP price index)}
\item{u}{unemployment rate (seasonally adjusted civilian unemployment rate, all civilian workers aged 16 years or older)}
\item{r}{interest rate (yield on the three-month Treasury bill rate)}
}
}
}
\source{
Koop, G. and Korobilis, D. (2010). Bayesian multivariate time series
methods for empirical macroeconomics. \emph{Foundations and Trends in Econometrics}, 3(4), pp. 267-358.
}
\usage{
KoopKorobilis2010
}
\description{
Quarterly US macroeconomic data (1953Q1-2006Q3) from Koop and Korobilis (2010)
}
\keyword{datasets}
